
//=====================================================
// Amaterske SQM - Sky Quality Meter
//   AstroMiK
//
//
// Detaily (schema, plosnak, navod, fotky):
//           http://sqm.astromik.org
//
#define verzeSW "2024-12-10"
//=====================================================



// volba typu senzoru teploty a vlhkosti (zvolit jen 1 ze 4 moznosti)
//#define senzor_DHT11                                   // DHT-11: nepresne cidlo teploty a vlhkosti
//#define senzor_DHT22                                   // DHT-22: trochu lepsi cidlo teploty a vlhkosti
#define senzor_BME                                       // BME280: presne cidlo teploty, tlaku a vlhkosti
//#define senzor_nic                                     // bez cidel (bude se merit jen plosny jas)

// volba typu senzoru naklonu / kompasu
#define modul_LSM303DLHC                                 // naklonomer s kompasem LSM303DLHC
//#define naklon_nic                                     // zadny modul pro mereni naklonu

#define pouzit_modbus                                    // misto puvodni verze predavani dat pres RS485 se pouzije klasicky modbus


// volba zobrazeni (zvolit jen 1 ze 2 moznosti)
#define displej_TM1637_5                                 // LED displej 5x7 segmentu (TM1637)
//#define signal_LED                                     // misto displeje jen signalizacni LED

#define internal_GPS  true                               // odkomentovat v pripade pouziti vnitrni GPS
//#define internal_GPS  false                            // odkomentovat v pripade, ze vnitrni GPS pouzita nebude (zustava moznost externi GPS)

#define pouzivat_SD_kartu                                // zakomentovanim je mozne vypnout pouzivani SD karty (zustane jen EEPROM a seriova linka + pripadne displej)
                                                         // zrusi se tim varovna hlaseni o nezasunute karte pri startu programu a pri pokusu o zapis (zapisy budou uplne zrusene)

// vybrat jazyk
#define CZ_language                                      // cestina
//#define EN_language                                    // anglictina jeste neni prelozena (texty jsou v souboru "jazyky.h")     //!!


// nastaveni analogovych urovni pro test baterie (meri se za odporovym delicem R7, R8, R6)
//     merici rozsah je 0 az 3,3V a to odpovida cislu 0 az 4095.
//  Uvedene meze vychazi ze skutecne zjistenych hodnot - v priloze je dokument "test_baterie.xls"
#define bat_level_3     620                              // (7V) nad level_3 je vyborny stav baterie                        (blika   1 tecka na displeji)
#define bat_level_2     528                              // (6V) mezi level_3 a level_2 (7V az 6V)je dobry stav baterie     (blikaji 2 tecky na displeji)
#define bat_level_1     441                              // (5V) mezi level_2 a level_1 (6V az 5V) je stredni stav baterie  (blikaji 3 tecky na displeji)
                                                         // pod mezi level_1 (pod 5V)                                       (blikaji 4 tecky na displeji)
#define ref_level      2401                              // kdyz je napajeni tak nizke, ze referencni napeti prekroci tuto uroven, rozblika se vsech 5 tecek

                                                         // kvuli uspore mista v PROGMEM se mohou nektere nepouzivane funkce uplne zrusit.
                                                         //       U funkci, ktere se maji omezit nebo vypnout se zrusi komentar (dve lomitka na zacatku radky)
//#define omezeni_funkci_1                                 //  toto omezeni zrusi moznost kalibrace urovni bat_level po spojeni bocnich pinu PA13(SWDIO) a 3V3
//#define omezeni_funkci_2                                 //  toto omezeni zrusi logovani nastaveni vnitrniho RTC (funkce "@>" a "@/")
//#define omezeni_funkci_3                                 //  toto omezeni zrusi experimentalni komunikaci s programem od Unihedronu (prikazy "rx", "R" a "ix")
//#define omezeni_funkci_4                                 //  toto omezeni vypne logovani zmen systemovych parametru na SD kartu
//#define omezeni_funkci_5                                 //  toto omezeni zrusi funkce expanzni desky pro vice cidel
//#define omezeni_funkci_6                                 //  toto omezeni zrusi vypisy nekterych textovych hodnot pres linku RS485 (elevace Slunce, elevace Mesice, osvetleni Mesice) 
//#define omezeni_funkci_7                                 //  toto omezeni zrusi moznost nastavit domaci souradnice pro astro vypocty primo v terenu podle GPS


// V minimalni variante se do EEPROM uklada techto 8 bajtu:
//     1. bajt       - znacka obsazeneho nebo volneho mista, znacka SEC/SELC, index prepnuteho cidla na rozsirujici desce
//     2. az 5. bajt - casovy udaj v sekundach od 1.1.1970
//     6. a 7. bajt  - udaj o namerenem plosnem jasu v tisicinach mag/arcsec2
//     8. bajt       - popis co zpusobilo spusteni mereni (tlacitko, seriova linka, automat, kalibrace), znacka stability mereneho jasu, adresa zarizeni pro komunikaci

// volitelne polozky, ktere se budou ukladat do EEPROM pri kazdem zaznamu (vice polozek znamena, ze se do EEPROM vejde mene zaznamu)
// --- PO ZMENE UKLADANYCH POLOZEK JE NUTNE PROVEST "HARD" FORMAT EEPROM (funkce "#FH" pres seriovou linku) ---
#define ukladat_teplotu             // do EEPROM se bude ukladat informace o teplote                                                           (plus  2 bajty na zaznam)
#define ukladat_tlak                // do EEPROM se bude ukladat informace o tlaku                                                             (plus  2 bajty na zaznam)
#define ukladat_vlhkost             // do EEPROM se bude ukladat informace o vlhkosti                                                          (plus  2 bajty na zaznam)
#define ukladat_infra               // do EEPROM se bude ukladat informace o INFRA slozce svetla                                               (plus  2 bajty na zaznam)
#define ukladat_full                // do EEPROM se bude ukladat informace o FULL slozce svetla                                                (plus  2 bajty na zaznam)
#define ukladat_contr_reg           // do EEPROM se bude ukladat informace o nastaveni Controll registru cidla svetla (zesileni, doba snimani) (plus  1  bajt na zaznam)
#define ukladat_GPS                 // do EEPROM se bude ukladat informace o zemepisnych souradnicich (nutny pridavny hardware)                (plus 10 bajtu na zaznam)
#define ukladat_naklon              // do EEPROM se bude ukladat informace o naklonu (nutny pridavny hardware)                                 (plus  2 bajty na zaznam)
#define ukladat_azimut              // do EEPROM se bude ukladat informace o naklonu (nutny pridavny hardware)                                 (plus  2 bajty na zaznam)
#define ukladat_astro               // do EEPROM se bude ukladat informace o elevaci Slunce a Mesice a o jeho osvetleni                        (plus  3 bajty na zaznam)

// priklad: Pro uplne ukladani vsech polozek bude 1 zaznam dlouhy 36 bajtu. Do 128kB EEPROM se tak vejde asi 3500 zaznamu.
//          Pro minimalni verzi ukladani (uklada se jen nezbytne minimum informaci a svetlo) je jeden zaznam dlouhy 8 bajtu a do stejne EEPROM se tak vejde pres 16000 zaznamu

#define posun_tlaku         60000                        // aby se udaj o zmerenem atmosferickem tlaku vesel do rozsahu 2 bajtu, je nutne odecitat nejakou konstantni hodnotu
#define offset_kompasu       +0.0                        // pripadne doladeni azimutu pri nepresne nalepenem cidle LSM303 hodnota se zadava jako float +/- v desetinnem tvaru

#define pocet_polozek_v_menu    27                       // pocet polozek v menu

float lux_kal_a, lux_kal_b, lux_kal_c;                   //   kalibracni parametry pro prevod hodnoty z cidla luxmetru na zobrazovanou hodnotu v luxech (zatim vubec netusim, jaky tvar bude mit prevodni krivka)
                                                         //      (v pripade linearniho posunuti budou pouzity pouze 2 parametry, jestli bude nejake prohnuti, pouzije se i treti parametr)
                                                         
#define verzePCB2                                        // pro druhou verzi plosnaku funguje trochu jinak mereni napeti baterie (testuje se navic jeden analogovy pin s referencnim napetim)

boolean menu_enable[33];                                 // 32 bitu pro zapinani a vypinani jednotlivych polozek v menu

byte delka_zaznamu;                                      // podle zvolenych polozek pro ukladani do EEPROM se nize vypocte delka jednoho zaznamu
unsigned long adr_dalsiho_zaznamu;                       // aby nebylo nutne pred kazdym ukladanim do EEPROM hledat volne misto, zapamatuje se do teto promenne adresa pristiho volneho prostoru
unsigned long adr_posl_zaznamu;                          // adresa, kam byl ulozen posledni zaznam (pro jednodussi ziskavani dat z EEPROM)

unsigned long max_zaznam_EEPROM;                         // adresa, na ktere je ulozeny prvni bajt z posledniho zaznamu (vypocitava se z velikosti pouzitelne EEPROM a z delky zaznamu)

#define min_EEPROM     700UL                             // Na jake adrese v EEPROM zacina oblast pro namerena data:
                                                         //     do adresy 20 jsou systemove parametry,
                                                         //     do adresy 50 je kalibracni tabulka pro teploty
                                                         //     od 50 do 500 jsou kalibracni tabulky pro cidla jasu (pro kazde ze 7 cidel zvlast)
                                                         //     od 500 do 575 je oblast pro 5 prednastavenych pozorovacich stanovist
                                                         //     do 600 nejake kalibrace pro luxmetr a oznaceni casove zony
                                                         //     do 612 kalibrace kompasu 
#define max_EEPROM    0x1FFFFUL                          // maximalni mozna adresovatelna bunka v EEPROM  (pro 128k x 8b EEPROM )

#define eeaddr_GEO_lat              485UL    // (2 bajty)   adresa v EEPROM se zemepisnou sirkou pro astronomicke vypocty
#define eeaddr_GEO_lon              487UL    // (2 bajty)   adresa v EEPROM se zemepisnou delkou pro astronomicke vypocty

#include <Wire.h>                                        // knihovna pro I2C komunikaci
#include <libmaple/iwdg.h>                               // podpora pro Watchdog
#include "jazyky.h"

#define ID1 (*(uint32_t *)0x1FFFF7E8)                    // unikatni identifikace procesoru
#define ID2 (*(uint32_t *)0x1FFFF7EC)
#define ID3 (*(uint32_t *)0x1FFFF7F0)
char ID_char[] = "xxxxxxxxxxxxxxxxxxxxxxxx\0";

#define ADC_CR2 (*(uint32_t *)0x40012408)                // ADC control registr

#if defined senzor_DHT11 || defined senzor_DHT22
  #include <OneWireSTM.h>                                // knihovna pro OneWire komunikaci (cidlo vlhkosti)
  #include <DHT.h>                                       // knihovna pro cidlo vlhkosti a teploty DHT11 / DHT22
  #define pin_DHT             PB14
#endif


#ifdef senzor_DHT11
    DHT cidlo_DHT(pin_DHT, DHT11);                       // inicializace DHT11
#endif
#ifdef senzor_DHT22
    DHT cidlo_DHT(pin_DHT, DHT22);                       // inicializace DHT22
#endif

#ifdef senzor_BME
  #include <Adafruit_BME280.h>                           // knihovna pro cidlo vlhkosti, teploty a tlaku
  Adafruit_BME280 cidlo_BME; // I2C
#endif


#include "RTClock.h"

RTClock rtclock (RTCSEL_LSE);
time_t tt;
time_t tt_file;
tm_t mtt;
tm_t mtt_file;
tm_t mtt_EEPROM;




#define pin_SD_CS           PA4                          // CS na SD karte

#define pin_tl_up          PA15                          // ovladaci tlacitka
#define pin_tl_dn           PB3
#define pin_tl_ok           PB4

#define pin_bzuk            PB1                          // pin pro piskak (pasivni)
#define pin_DERE           PB15                          // pin pro prepinani smeru vysilani 


#define pin_dis_CLK        PB12                          // piny pro komunikaci s displejem
#define pin_dis_DIN        PB13

#ifdef signal_LED
  #define pin_LED_R          PA1                         // cervena LED pro verzi bez displeje
  #define pin_LED_G          PB8                         // zelena LED pro verzi bez displeje
  #define pin_LED_B          PB9                         // modra LED pro verzi bez displeje
#endif


#define pin_karta_IN       PB10                          // pin pro test zasunute karty ve druhe verzi plosnaku
#define pin_refu           PB0                           // pin pro referencni napeti pro kontrolu stavu baterie


#define pin_AD_BAT         PA0                           // pin pro testovani napeti baterie
#define pin_TS            PA14                           // specialni pin pro zaznam casoveho razitka (TimeStamp)

#define pin_TEST_HW       PA13                           // spusteni testu bez pripojene komunikace



#define pin_EXT_ZAP       PB11                           // pin pro zjistovani pripojeneho EXT zarizeni (GND = pripojeno)


#define I2C_ADDR_EEPROM_RW_0     0b1010000               // I2C adresa EEPROM pro cteni a zapis dat pro bunky 0 az 65535 (adresovaci piny v LOW)
#define I2C_ADDR_EEPROM_RW_1     0b1010001               // I2C adresa EEPROM pro cteni a zapis dat pro bunky nad 65535  (adresovaci piny v LOW)
byte I2C_ADDR;                                           // podle aktualni adresy zapisovane, nebo ctene bunky se nastavuje I2C adresa EEPROM

#define I2C_ADDR_EXPA    0x20                            // I2C adresa expanderu 0x20 (vsechny adresovaci piny v LOW)
#define I2C_ADDR_GPS     0x17                            // I2C adresa pridavneho modulu pro GPS


#ifdef modul_LSM303DLHC
  #define I2C_ADDR_KOMPAS       0b0011110
  #define I2C_ADDR_NAKLON       0b0011001
  #define  OUT_X_L_A            0x28
  #define  CTRL_REG1_A          0x20
  #define  CTRL_REG4_A          0x23
  #define  CRA_REG_M            0x00
  #define  CRB_REG_M            0x01
  #define  MR_REG_M             0x02
  #define  DLHC_OUT_X_H_M       0x03
#endif


// Modbus
#include <ModbusRtu_AstroMiK.h>                         // knihovna pro Modbus komunikaci (upravena puvodni knihovna )
Modbus MBslave(1, Serial2, pin_DERE);                   // ID = 1 (bude dale nastaven), ser.port, ridici pin (pin_DERE)
uint16_t MODdata[60];                                   // tabulka hodnot parametru Modbus
byte stav_MOD = 0;
byte posledni_funkce;



boolean leto =  false;                                   // neni letni cas

byte data;
boolean open_OK;                                         // pomocna promenna, jestli funguje zapis na kartu (jestli je zasunuta karta)

byte prumery;                                            // globalni promenna pro pocet mereni hodnot svetla, ze kterych se vypocte prumer
byte automat;                                            // globalni promenna pro nastaveni intervalu mezi automatickym merenim svetla v minutach (0=automat vypnuty)
byte selc;                                               // globalni promenna pro nastaveni casove zony (0=SEC, 1=SELC)
byte slave_addr;                                         // SLAVE adresa pro komunikaci                     
int list;                                                // listovani v poslednich zmerenych hodnotach
byte hranice_ukladani;                                   // hodnota svetla v desetinach mag/arcsec2 pod kterou se nebudou ukladat zaznamy do EEPROM ani na kartu

unsigned long posledni_autospusteni;                     // pomocna promenna pro cas posledniho automatickeho spusteni

char prijato[16];                                        // pole, do ktreho se ukladaji znaky, prijate pres seriovou linku (nastaveni casu, zadost o informace, korekce ....)
unsigned int i;                                          // pomocna globalni promenna pro ruzne smycky

byte prijimaci_pole[16];                                 // pole, do ktereho se ukladaji pozadavky od MASTERa pri komunikaci RS485
byte vysilaci_pole[50];                                  // pole, do ktereho se pripravuji data k odeslani do MASTERa pri komunikaci RS485

byte dokoncene_mereni = 2;                               // na zacatku mereni se tato promenna vynuluje a po dokonceni mereni (ktere muze trvat treba i 20 sekund) se nastvi zpatky na 2
byte prubeh_mereni;                                      // Kdyz prijde dotaz na posledni namerene hodnoty ( pres seriovou linku RS485), ale mereni jeste nebude ukoncene, odesle se zbyvajici pocet mereni

unsigned long casova_znacka;                             // pouzito pro zjisteni doby, jak dlouho se ma ignorovat komunikace pres RS485
unsigned int ignor_cas = 3000;

byte pouzij_CRC;                                         // podle parametru v EEPROM[15] je mozne vypnout kontrolu CRC pri prijmu pozadavku pres RS485
byte index_cidla_svetla = 1;                             // v pripade rozsirujici desky pro vice cidel je v teto promenne ulozene cislo prave aktivniho cidla 1 az 7 - (specialni pripad 0)
byte pocet_cidel_svetla;                                 // pocet cidel svetla na expanzni desce


byte lap_count;                                          // pocitadlo mezicasu od spusteni stopek (1 az 99)

boolean blokuj_TS;                                       // zaznam casove znacky je blokovan az do uvolneni tlacitka (aby pri drzeni tlacitka nebylo mozne neustale ukladani casovych znacek)
boolean TS_lock;                                         // znacka, ze je bocni tlacitko zamcene proti nechtenemu sepnuti

byte alarm_bat_pamet = 1;                                // pouzito pro varovne akusticke signaly pri poklesu napeti pod nastavene meze

byte pole_EEPROM[37];                                    // globalni promenna, do ktere se pri kazdem mereni ukladaji namerena data. Toto pole se nasledne zapisuje do EEPROM a na SD kartu
byte pole_GPS_I2C[30];                                   // pole, do ktereho se stahuji data z GPS modulu pres I2C
char pole_GPS_NMEA[] = "$GPRMC,110109.00,A,4926.67723,N,01421.99214,E,0.521,,311220,,,A*72   + nejaka rezerva    @\0";   // pole, do ktereho budou stahovat neupravene NMEA zpravy z GPS modulu

unsigned int  int_svetlo;                                // globalni promenne, ktere uchovavaji posledni zmerene hodnoty
unsigned int  int_teplota;
unsigned int  int_vlhkost;
unsigned int  int_tlak;
unsigned int  int_infra;
unsigned int  int_full;
unsigned int  int_naklon;
unsigned int  int_azimut;
unsigned long int_luxy;
byte          byte_cnf;
unsigned long GPS_lat;
unsigned long GPS_lon;
unsigned int  GPS_alt;

unsigned long GPS_temp_time;                             // pri automatickem mereni se zapnutou a zafixovanou GPS se nenastavuje RTC, ale do zaznamu se pouziva prijaty cas z GPS
boolean GPS_RTC_flag;                                    // globalni promenna, ktera signaluzuje, ze se misto casu z RTC ma pouzit cas z GPS ('GPS_temp_time')
boolean vnitrniGPS = false;                              // typ pouzite GPS desky (urcuje se z prijatych dat z GPS modulu)

byte posledni_i2C_prikaz;                                // aby se zbytecne neustale neobnovoval stav LED v GPS modulu

byte         znacka;                                     // znacka, co spustilo mereni (tlacitko, automat , seriova linka) + dalsi info (SLAVE adresa a stabilita svetla)

int pamet_LED_svitilna = 1;                              // az do vypnuti napajeni se v teto promenne uchovavanastaveny jas cervene svitilny (0...zhasnuto az 5...sviti vsech 5 sedmisegmentovek)

// promenne pro pripadnou korekci vnitrniho RTC
unsigned long cas_minuleho_nastaveni;                    // cas predchoziho nastaveni casu v sekundach od 1.1.1970
unsigned long korekce;                                   // po jake dobe se ma pricist nebo odecist 1 sekunda

byte rozhod_stab;                                        // rozhodovaci uroven pro stabilni jas

byte D_pamet[5];                                         // V tomto poli se bitove uchovava aktualni stav vsech rozsvicenych segmentu
byte D_pam_pred_int[5];                                  //  podobne pole, ktere si pamatuje displej (zobrazenou polozku v menu) pred spustenim automatickeho mereni
byte pozice_tecky;                                       // Pocet desetinnych mist pro zobrazeni cisla (0 az 4) - na pozici 0 ale tecka nikdy nesviti
long posledni_n;                                         // Pri aktualizaci pozice desetinne tecky se znovu zobrazi posledni zadane cislo 

byte pom_disp_scroll[30];                                // pro pripad, kdy je nutne scrollovat nejaky text na displeji, se ten text uklada sem do toho pole

int LOC_hod, LOC_den, LOC_mes, LOC_min, LOC_sek;         // promenne, ktere obsahuji aktualni cas v RTC (mistni cas)
unsigned int LOC_rok ;                                   // rok (od 1970)
byte LOC_dvt;                                            // den v tydnu (po ... ne)
unsigned long rokmesden;                                 // rok mesic a den ve formatu yyyymmdd
unsigned int mesden;                                     // mesic a den ve formatu mmdd

boolean RTC_SD_DEBUG;                                    // pokud je tato znacka aktivni, budou se pri kazdem nastaveni hodin pres GPS zapisovat do zvlastniho souboru informace o presnosti RTC

int LOC_den_file, LOC_mes_file;                          // promenne, ktere obsahuji datum pro nazev slozky na SD karte
unsigned int LOC_rok_file ;                              // rok (od 1970)

unsigned long ctdwn_end;                                 // cas v sekundach od 1.1.1970, kdy se ma spustit alarm po odpoctu
unsigned long dvouminutovy_bzuk=0;                       // pipani pri alarmu bude trvat maximalne 2 minuty. Sem se ulozi cas, ve kterem pipani zacalo

                  // definice znaku pro displej se spolencou anodou
                  //       0    ,       1    ,       2    ,       3    ,       4    ,       5    ,       6    ,       7    ,       8    ,       9    
byte graf_def[10] = {0b00111111 , 0b00000110 , 0b01011011 , 0b01001111 , 0b01100110 , 0b01101101 , 0b01111101 , 0b00000111 , 0b01111111 , 0b01101111 };

byte pole_citlivosti[4] = {0 , 16 , 32 , 48 };           // stupen citlivosti cidla
int posledni_citlivost = 3;                              // posledni pouzity stupen citlivosti (pouziva se k uychleni mereni svetla pri vice merenich v rade)


unsigned long pole_baudrate[4] = {9600 , 19200 , 38400 , 115200 };      // prednastavene komunikacni rychlosti


byte znacka_listu;                                       // pri vypisu svetla se jeste pred datumem vypise informace o typu zaznamu:
                                                         //   'c' = jeden z 10 kalibracnich vzorku
                                                         //   'C' = prumer vsech 10 kalibracnich vzorku
                                                         //   'u' = spusteno tlacitkem "Up"
                                                         //   'd' = spusteno tlacitkem "Down"
                                                         //   'o' = spusteno tlacitkem "Ok"
                                                         //   'A' = spusteno automatem
                                                         //   'M' = spusteno komunikaci


byte znak_stupen = 32;

int kal_svetlo[20][2] ;                                  // pole kalibracnich hodnot pro svetlo
unsigned int kal_teplota[10][2];                         // pole kalibracnich hodnot pro teplotu
int kal_serial[20][2] ;                                  // pres seriovou linku se nejdriv ulozi prijata data do tohoto pole. Po jejich prekontrolovani se pak prepisou do jednoho z poli kal_teplota[], nebo kal_svetlo[]


//  index pole 100x                                                                                                            111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111122222222222222222222222222222222222222222222222222222222
//  index pole 10x                   111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999000000000011111111112222222222333333333344444444445555555555666666666677777777778888888888999999999900000000001111111111222222222233333333334444444444555555
//  index pole 1x          0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345    
//                         --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
char vystupni_retezec[] = "                                                                                                                                                                                                                                                               \0";
// priklad                " 00018 ; 000823 ; 1609232363 ; 29.12.2020 ; 08:59:23 ; Ut ;  SEC ; 08881 ; 08,881 ; 01 ; 01 ; komun ;  STABIL  ; 17989 ; 48311 ; 600 ;  428x ; 07196 ; +21,96 ; 05151 ; 051,51 ; 093410 ; +49,457154 ; +120,457156 ; -1234 ; +165.8 ; 123 ;  -45 ;  -27 ;  58  \0";    

char DATCAS_retezec[] = "2022/01/24;18:45:00";           // pomocny globalni retezec pro predavani datumu a casu

boolean mute_tone_bit;                                   // bit5 na adrese 15 umoznuje uplne vypnout pipani (nastavenim na HIGH)

unsigned long stisk_time;                                // pro pocitani delky stisku tlacitka
boolean stisk;                                           // pro pociteni delky stisku tlacitka
boolean bude_save;                                       // pri opusteni menu tato promenna urcuje, jestli se ma nova hodnota ulozit do EEPROM/(RTC), nebo jestli se nic ukladat nebude 
byte jas_displeje;                                       // globalni promenna pro nastaveni jasu displeje podle posledni zmerene hodnoty svetla

#define AUTOEXITMENU      300000UL                       //  po jake dobe necinnosti v [ms] se menu automaticky opusti (5 minut * 60 sekund * 1000 milisekund)
boolean auto_exit_menu;                                  // pri 'true' se okamzite vypadne ze vsech vnorenych podprogramu jakoby se stiskavalo kratce tlacitko OK az do doby vypadnuti do hlavni smycky
unsigned long auto_exit_casovani;                        // po dobehnuti teto promenne na hodnotu udanou v definici AUTOEXITMENU se automaticky vypadne z menu

byte spec_rezim = 0;                                     // podle pripojenych I2C periferii je mozne provozovat zarizeni v ruznych specialnich rezimech //!!(r30s)

boolean pole_zobr_menu[33];                              // nastavuje, ktere polozky se budou zobrazovat v menu ([0]=List; [1]=Auto; [2]=zona .... [13]=GPS)
boolean prvni_vstup_do_menu = true;                      // pri prvnim vstupu do menu po zapnuti napajeni se hleda nejnizsi zobrazitelna polozka v menu

byte oddelovaci_bajt;                                    // obsah EEPROM(9), ktery obsahuje informace o oddelovacich znaminkach pro CSV soubor na SD karte
boolean des_carka;                                       // oddelovac desetinnych mist (tecka = false; carka = true)
boolean uzavirat_uvozovky;                               // uzavirat kazdou hodnotu na SD karte do uvozovek
boolean vytvaret_hlavicku;                               // Na zacatek kazdeho souboru na SD karte zapise (nebo nezapise) textovy popis sloupcu
byte oddelovac_polozek;                                  // Urcuje znak, kterym se budou oddelovat jednotlive polozky v CSV souboru (mezera, strednik, carka, tabulator)

char oddelovac;                                          // znak pro oddelovani polozek v zaznamech (strednik, carka, mezera, tabulator)
char uvozovky;                                           // znak pro uzavirani polozek do uvozovek (nebo mezera)
char desod;                                              // znak pro oddeleni desetinnych mist (carka, nebo tecka)

byte pocet_bzuku;                                        // Vypinani akusticke signalizace chyby po 5 bzuknutich

int styl_vodovahy;                                       // 0 ... elevace; 1 ... dvouosa vodovaha horizontalne nebo svisle (podle aktualniho uhlu naklonu)
int astro_parametr;
boolean stabilni_naklon;                                 // znacka, jesli posledni mereni naklonu vratilo stabilni hodnotu, nebo jestli bylo posledni mereni nestabilni

double GeoLon;                                           // souradnice pozorovaciho mista pro astro vypocty polohy Slunce a Mesice
double GeoLat;

int    astro_UTC_min;
int    astro_UTC_rok;
int    astro_UTC_mes;
int    astro_UTC_den;
int    astro_UTC_hod;
int    casova_zona;

int letni_posun;                                         // casovy posun mistni casove zony proti UTC v lete (SELC/CEST = 2) je ulozeny v EEPROM (489)
int zimni_posun;                                         // casovy posun mistni casove zony proti UTC v zime ( SEC/CET  = 1) je ulozeny v EEPROM (490)

char char_zima[5];                                       // 4 uzivatelsky nastavene znaky pro oznaceni zimni casove zony + '\0'
char char_leto[5];                                       // 4 uzivatelsky nastavene znaky pro oznaceni zimni casove zony + '\0'
char char_Xzima[5];                                      // stejne jako char_zima, akorat prvni znak je nahrazen podtrzitkem
char char_Xleto[5];                                      // stejne jako char_leto, akorat prvni znak je nahrazen podtrzitkem

float  Mes_elevace;
float  Slu_elevace;
double Mes_osvit;
float  Mes_azimut;
float  Slu_azimut;

int zadana_rektascenze;
int zadana_deklinace;
float obecna_elevace;
float obecny_azimut; 

int lux_citlivost;                                       // pro automaticke urcovani citlivosti cidla v rezimu luxmetr
long lux_klouzak[6];                                     // pro klouzave prumerovani poslednich 5 mereni
byte lux_index_klouzak;                                  // ukazatel po pole klouzaku pro luxometr

unsigned int index_naklonu;                              // znacka, kam se v kazdem zaznamu uklada naklon
unsigned int index_azimutu;                              // znacka, kam se v kazdem zaznamu uklada azimut

//       ---- zacatek bloku R30s --

boolean blokuj_znacku30 = false;                         //!! pro spec rezim (r30s) - pri zobrazenem menu nebo nejakych jinych informacich na displeji nezobrazuje znacku na prvni segmentovce


//       ---- konec bloku R30s --

boolean predchozi_data_RDY;                              // jen pro logovani zapnuti vypinace u GPS modulu

#ifdef pouzivat_SD_kartu                                 // kdyz je nastaveno pouzivani SD karty, tak se muze ukladat

#include "SdFat.h"                                       // nova knihovna pro praci s SD kartou
#include "sdios.h"

SdFat sd;                                                // pro SD kartu
SdFile soubor;
#endif


//======================================================================================================================
void setup(void)
  {

    afio_cfg_debug_ports(AFIO_DEBUG_NONE);               // zprovozneni JTAG pinu jako obycejnych GPIO (PB3 = tlacitko "DOLU"; SWSCK = TimeStamp; SWDIO = HW test baterie )
    
    pinMode(pin_SD_CS, OUTPUT);

    pinMode(pin_tl_up, INPUT_PULLUP);
    pinMode(pin_tl_dn, INPUT_PULLUP);
    pinMode(pin_tl_ok, INPUT_PULLUP);

#if internal_GPS
    pinMode(pin_EXT_ZAP, INPUT);                        // pri pouziti vnitrni GPS se pin pri vypnute GPS pridrzuje odporem v LOW
    predchozi_data_RDY = LOW;    
#else
    pinMode(pin_EXT_ZAP, INPUT_PULLUP);                 // pri pouziti vnejsi GPS se nezapojeny pin pridrzuje odporem v HIGH
    predchozi_data_RDY = HIGH;    
#endif

    pinMode(pin_bzuk,OUTPUT);

    pinMode(pin_dis_CLK, OUTPUT);
    pinMode(pin_dis_DIN, OUTPUT);
    pinMode(pin_DERE, OUTPUT);





#ifdef signal_LED
    pinMode(pin_LED_R, OUTPUT);
    analogWrite(pin_LED_R , 255);                        // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
    pinMode(pin_LED_G, OUTPUT);
    analogWrite(pin_LED_G , 255);                        // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
    pinMode(pin_LED_B, OUTPUT);
    analogWrite(pin_LED_B , 255);                        // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
#endif  

    digitalWrite(pin_dis_CLK , LOW);
    digitalWrite(pin_dis_DIN , LOW);


    pinMode(pin_karta_IN, INPUT_PULLUP);
    pinMode(pin_AD_BAT  , INPUT_ANALOG);
    pinMode(pin_TEST_HW , INPUT_PULLDOWN);               // pin pro specialni testy urovni napajeciho napeti (kdyz nesmi byt zapojena komunikace)
    pinMode(pin_refu    , INPUT_ANALOG);
    pinMode(pin_TS      , INPUT_PULLUP);
    

    Wire.begin();
    delay(100);

    pocet_bzuku = 5;
    jas_displeje = 3;

// vypocet delky zaznamu podle zvolenych polozek pro ukladani do EEPROM
    delka_zaznamu = 8;
#ifdef ukladat_teplotu    
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_tlak       
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_vlhkost    
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_infra      
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_full       
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_contr_reg 
    delka_zaznamu = delka_zaznamu + 1;
#endif

#ifdef ukladat_GPS 
    delka_zaznamu = delka_zaznamu + 10;
#endif

#ifdef ukladat_naklon 
    index_naklonu = delka_zaznamu;
    delka_zaznamu = delka_zaznamu + 2;
#endif

#ifdef ukladat_azimut 
    index_azimutu = delka_zaznamu;
    delka_zaznamu = delka_zaznamu + 2;
#endif

#ifdef ukladat_astro 
    delka_zaznamu = delka_zaznamu + 3;
#endif

    // nastaveni komunikacni rychlosti podle bitu 4 a 5 na adrese 14 v EEPROM
    Serial.begin(pole_baudrate[(EEPROM_read(14) & 0b00110000) >> 4]);            // servisni a nahravaci UART (piny PA9, PA10) 
    Serial2.begin(pole_baudrate[(EEPROM_read(14) & 0b00110000) >> 4]);           // RS485  (piny PA2, PA3)

    // vypocet adresy prvniho bajtu posledniho zaznamu
    max_zaznam_EEPROM = min_EEPROM + ((max_EEPROM - min_EEPROM - delka_zaznamu) / delka_zaznamu) * delka_zaznamu;
    
    byte zachytny_bod_hledani = EEPROM_read(18);
    adr_dalsiho_zaznamu = najdi_prazdne_misto(zachytny_bod_hledani);             // pri zapnuti se musi prohledat cela pamet od posledniho zachytneho bodu, ale priste uz se bude brat prazdny prostor z hodnoty teto promenne
    

    automat = EEPROM_read(10);                           // v EEPROM se na adrese 10 nachazi cislo, ktere udava, v jakych intervalech (minuty) se ma merit jas povoleny rozsah 0 az 255 (0=automat vypnuty)

    prumery = EEPROM_read(11);                           // v EEPROM se na adrese 11 nachazi cislo, ktere udava, kolik se ma prumerovat hodnot povoleny rozsah 1 az 20
    if (prumery == 0 or prumery > 20)                    // kdyz je tam nejaky nesmysl, tak se provede kompletni prepsani defaultnich parametru systemu do EEPROM
      {
        default_parametry();
        default_kalibrace_svetla();
        default_kalibrace_teploty();
        delay(1000);
        SOFT_RESET();
      }

    hranice_ukladani = EEPROM_read(19);                  // od jake urovne svetla se maji ukladat zaznamy do EEPROM


    zimni_posun = EEPROM_read(489) - 100;
    letni_posun = EEPROM_read(490) - 100;

    for (int i = 575; i < 579 ; i++)                     // uzivatelsky definovane popisy casovych zon jsou ulozene v EEPROM
      {
        char_zima[i-575] = EEPROM_read(i);
        char_leto[i-575] = EEPROM_read(i+4);
        char_Xzima[i-575] = EEPROM_read(i);
        char_Xleto[i-575] = EEPROM_read(i+4);
      }
    char_zima[4] = '\0';
    char_leto[4] = '\0';
    char_Xzima[4] = '\0';
    char_Xleto[4] = '\0';
    char_Xzima[0] = '_';                                 // pro vypisy smazanych zaznamu z EEPROM se vypisuje oznaceni zony s prvnim znakem nahrazenym podtrzitkem
    char_Xleto[0] = '_';

      
    slave_addr = (0b00001111 & EEPROM_read(14));         // v EEPROM se na adrese 14 nachazi SLAVE adresa pro komunikaci (bity 0 az 3). Na bitech 4 a 5 je komunikacni rychlost.
    if (slave_addr == 0 or slave_addr > 15)              // kdyz je tam nejaky nesmysl, tak se prepise na 1
      {
        slave_addr = 1;
        EEPROM_write(14,1);                              // komunikacni rychlost se v tom pripade take zmeni na defaultni 9600
      }

    rozhod_stab = EEPROM_read(16) ;


#ifdef pouzit_modbus
    MBslave.setID(slave_addr);              // ID pro modbus
    MBslave.start();


    MODdata[31] = 0;                        // status mereni na 0 (mereni dokonceno)

    MODdata[27] = 0xFFFF;                   // [30028] rezerva
    MODdata[28] = 0xFFFF;                   // [30029] rezerva
    MODdata[29] = 0xFFFF;                   // [30030] rezerva
    MODdata[30] = 0xFFFF;                   // [30031] rezerva

    uint16_t index_posledniho_zaznamu = ((adr_dalsiho_zaznamu - min_EEPROM) / delka_zaznamu);
    MODdata[39] = index_posledniho_zaznamu; // [40038] index posledniho zaznamu

    MODdata[40] = prumery;                  // [40041] pocet prumerovani
    MODdata[41] = zimni_posun;              // [40042] zimni posun casu 
    MODdata[42] = letni_posun;              // [40043] letni posun casu 
    MODdata[43] = rozhod_stab;              // [40044] rozhodovaci uroven  pro stabilni jas
    MODdata[44] = hranice_ukladani;         // [40045] hranice jasu pro ukladani do EEPROM
    MODdata[45] = automat;                  // [40046] interval automatickeho spousteni

    MODdata[46] = 0xFFFF;                   // [40047] rezerva
    MODdata[47] = 0xFFFF;                   // [40048] rezerva
    MODdata[48] = 0xFFFF;                   // [40049] rezerva
    MODdata[49] = 0xFFFF;                   // [40050] rezerva
    
    MODdata[50] = 0xFFFF;                   // [40051] adresa EEPROM pro funkci pristupu do systemu (0 az 699)
    MODdata[51] = 0xFFFF;                   // [40052] obsah EEPROM pro funkci pristupu do systemu  (0 az 255)

    MODdata[58] = 0;                        // vymazani posledniho modbus prikazu

    if (adr_dalsiho_zaznamu > min_EEPROM)                                      // kdyz nejaky starsi zaznam existuje, tak se pripravi pro stazeni pres MODBUS
      {
        for (byte i = 0 ; i < delka_zaznamu ; i++)
          {
            pole_EEPROM[i] = EEPROM_read(adr_dalsiho_zaznamu - delka_zaznamu  + i);
          }
        EEPROM_to_MOD(index_posledniho_zaznamu);
      }


#endif



    pouzij_CRC = EEPROM_read(15) & 0b00000011;           // z adresy 15 se pouzivaji pro CRC jen 2 nejnizsi bity (nejvyssi bit je 'TS_lock')

    if ((EEPROM_read(15) & 0b01000000) == 0)  RTC_SD_DEBUG = false;              // zapis do souboru "RTC_set.csv" zakazan
    else                                      RTC_SD_DEBUG = true;               // zapis do souboru "RTC_set.csv" povolen


    mute_tone_bit = ((EEPROM_read(15) & 0b00100000) >> 5);                       // bitem 5 nastavenym na HIGH je mozne kompletne vypnout vsechna pipani pres funkci tone()
    
    pocet_cidel_svetla = EEPROM_read(17) ;
    
    
    selc = EEPROM_read(8);                               // v EEPROM se na adrese 8 nachazi casova zona (0=SEC, 1 = SELC)
    if (selc > 1) 
      {
        selc = 1;
        EEPROM_write(8,selc);
      }
    if (selc == 0) leto = false;
    else           leto = true;

    oddelovaci_bajt = EEPROM_read(9);
    des_carka         = bitRead(oddelovaci_bajt,2);
    uzavirat_uvozovky = bitRead(oddelovaci_bajt,3);
    vytvaret_hlavicku = bitRead(oddelovaci_bajt,4);
    oddelovac_polozek = oddelovaci_bajt & 0b00000011;

    if (des_carka == true)  desod = ',';
    else                    desod = '.';

    if (uzavirat_uvozovky == true)  uvozovky = '"';
    else                            uvozovky = ' ';     


    ctdwn_end = EEPROM_read_long(495);

    float geopomprom = EEPROM_read_int(eeaddr_GEO_lat);
    if (geopomprom > 32767) geopomprom = geopomprom - 65536  ; 
    GeoLat = geopomprom / 10.0;
    
    geopomprom = EEPROM_read_int(eeaddr_GEO_lon);
    if (geopomprom > 32767) geopomprom = geopomprom - 65536  ; 
    GeoLon = geopomprom / 10.0;


    if (GeoLat > 90 or GeoLat < -90 or GeoLon > 180 or GeoLon < -180)            // souradnice jsou uplne mimo, provede se default na 50/15
      {
        EEPROM_write_int(   eeaddr_GEO_lat   ,         500);        // zemepisna sirka pro astro vypocty na 50.0 stupnu
        EEPROM_write_int(   eeaddr_GEO_lon   ,         150);        // zemepisna delka pro astro vypocty na 15.0 stupnu
      }

    // po resetu je mozne obnovit defaultni nastaveni promennych v EEPROM (bez kalibrace) dlouhym stiskem vsech 3 tlacitek zaroven
    if (digitalRead(pin_tl_up) == LOW and digitalRead(pin_tl_dn) == LOW and digitalRead(pin_tl_ok) == LOW) 
      {
        tone_X(pin_bzuk,4000, 10);
        unsigned long starttime = millis();
        while (digitalRead(pin_tl_up) == LOW and digitalRead(pin_tl_dn) == LOW and digitalRead(pin_tl_ok) == LOW)
          {
            delay(50);
            if (millis() - starttime > 10000UL)
              {
                tone_X(pin_bzuk,4000, 10);
                delay(100);
                default_parametry();

#ifdef displej_TM1637_5
                zobraz_text(42);                         // dEFA
                delay(500);                
                zobraz_text(28);                         // prazdny displej
#endif                
#ifdef signal_LED
                LED_R_onoff(true);
                LED_G_onoff(true);
                LED_B_onoff(true);
                delay(500);                
                LED_R_onoff(false);
                LED_G_onoff(false);
                LED_B_onoff(false);
                
#endif
                
                break;
              }
            else
              {
                graf_natazeni(millis() - starttime , 666);    // vystrazny graf z pomlcek na displeji - zaplneni celeho displeje trva asi 10 sekund
              }
          }
        while (digitalRead(pin_tl_up) == LOW or digitalRead(pin_tl_dn) == LOW or digitalRead(pin_tl_ok) == LOW)
          {
            tone_X(pin_bzuk,3000, 10);
            delay(500);
          }
        delay(100);

        delay(1000);
        SOFT_RESET();
      
      }

#ifdef pouzivat_SD_kartu                                 // kdyz je nastaveno pouzivani SD karty, ale karta neni zasunuta, zobrazi se pri startu na sekundu varovani, nebo blikne modra LED
    if (digitalRead(pin_karta_IN)== HIGH)                // karta vysunuta
      {
        chyba(3);                                        // No-SD
      }
#endif

    // test funkcnosti RTC (pocet sekund od 1.1.1970 musi byt vetsi, nez interval do 1.1.2020)
    unsigned long tt = rtclock.now();
    if (tt < 1577833000UL or tt > 3000000000UL)          // datum v RTC je drivejsi nez rok 2020, nebo pozdejsi nez 2065
      {
        chyba(4);                                        // chyba RTC
        if (EEPROM_read_long(0) != 0)             EEPROM_write_long(0,0);                    // zruseni znacky posledniho nastaveni casu
        if (EEPROM_read_long(4) != 0x7FFFFFFFUL)  EEPROM_write_long(4,0x7FFFFFFFUL);         // zruseni korekce
        
      }

    // aby se nemuselo pri kazdem pozadavku o cas z RTC  sahat do EEPROM, nastavi se nasledujici globalni promenne jen pri startu, nebo pri dalsim serizeni RTC
    cas_minuleho_nastaveni = EEPROM_read_long(0);        // Z EEPROM se zjisti, kdy se naposled serizoval cas (cas je v sekundach od 1.1.1970)
    korekce                = EEPROM_read_long(4);        // a jaka byla pri te prilezitosti vypoctena korekce (za jak dlouho se pricist, nebo odecist extra sekunda)


    // Do funkce 'STOPKY' se da vstoupit hned po zapnuti napajeni a soucasnem stisku tlacitka OK
    if (digitalRead(pin_tl_up) == HIGH and digitalRead(pin_tl_dn) == HIGH and digitalRead(pin_tl_ok) == LOW) 
      {
        stopky();
      }  
    test_nulovani_citace();                              // nulovani citace casovych razitek a prednastaveneho stanoviste po kazdem poledni

    if (digitalRead(pin_TS) == HIGH)                     // kdyz je pri zapnuti napajeni TS tlacitko uvolnene (normalni zapnuti posuvnym prepinacem)
      {
        TS_zamek(false);                                 // zamek TS tlacitka se automaticky odblokuje
      }

    // SW blokovani bocniho tlacitka je mozne pri zapnuti napajeni a soucasnem stisku tlacitka DOLU
    if (digitalRead(pin_tl_up) == HIGH and digitalRead(pin_tl_dn) == LOW and digitalRead(pin_tl_ok) == HIGH) 
      {
        TS_zamek(true);
        zobraz_text(36);                                 // "-Loc-"
        while (digitalRead(pin_tl_dn) == LOW) delay(100);          // ceka se na uvolneni tlacitka DOLU
        delay(100);
        zobraz_text(28);                                 // zhasne displej
      }

    if ((EEPROM_read(15) & 0b10000000) > 0)  TS_lock = true;       // podle hodnoty v EEPROM se nastavi globalni promenna pro zamek TS tlacitka
    else                                     TS_lock = false;


#ifdef modul_LSM303DLHC                                  // pokud je zapojeny naklonomer, provede se jeho zakladni nastaveni a nacteni kalibrace
    naklon_setup();
    kompas_setup();
#endif 

#ifdef senzor_BME
    cidlo_BME.begin(0x76);
#endif  

#if defined senzor_DHT11 || defined senzor_DHT22
    cidlo_DHT.begin();
#endif 


    unsigned long menubity = EEPROM_read_long(491);      // polozky v menu je mozne libovolne povypinat. V teto casti EEPROM je ulozena informace o tom, ktere polozky jsou viditelne (4 bajty = max 32 polozek)
    for (int polozka = 0 ; polozka <= pocet_polozek_v_menu ; polozka ++)
      {
        menu_enable[polozka] = bitRead(menubity, polozka);
      }

    if (digitalRead(pin_TS) == LOW and TS_lock == false) // pri zkratu pinu SCK a GND na bocnim konektoru desky WeAct (BluePill +) se zaznamena casove razitko
      {
        if (digitalRead(pin_tl_up) == LOW and digitalRead(pin_tl_dn) == HIGH and digitalRead(pin_tl_ok) == HIGH)   // kdyz je pri stisku bocniho tlacitka zaroven stisknuto UP, prejde se do volby viditelnych polozek v menu
          {
            #ifdef displej_TM1637_5                      // a samozrejme musi byt k dispozici sedmisegmentovy displej   
            editace_polozek_menu();
            #endif
          }
        else                                             // kdyz spolecne s bocnim tlacitkem neni stisknuto OK, zaznamena se normalne casove razitko
          {
            casove_razitko();                            // pri dlouhem drzeni se po nekolika sekundach zobrazi cas - soucast podprogramu 'casove_razitko()'
            delay(100);                                  // uvolnenim tlacitka v teto fazi se vypne napajeni a program skonci                
          }
      }

    if (digitalRead(pin_TS) == LOW and TS_lock == true)  // kdyz je TS tlacitko blokovane, zobrazi se jen kratke upozorneni na displeji a casova znacka se neuklada
      {
        chyba(7);                                        // pipnuti a zobrazeni napisu "-Loc-"       
        while(true)   asm("wfi") ;                       // nekonecna smycka s prechodem procesoru do nizke spotreby, ktera se prerusi az vypnutim napajeni
      }


    prijem();

#ifdef displej_TM1637_5
    zobraz_text(28);                                     // zhasne displej pro pripad, ze by neco zustalo v jeho pametech
#endif

#ifdef signal_LED                  
    LED_B_onoff(false);                                  // zhasne modrou LED
#endif


    
    // nacteni kalibracnich dat pro svetlo z EEPROM pro cidlo s indexem 1
    for (byte i = 0 ; i < 15 ; i++)
      {
        kal_svetlo[i][0] = EEPROM_read_int((i*4) + 50);
        kal_svetlo[i][1] = EEPROM_read_int((i*4) + 52);
      }

    // nasledujici 2 kalibracni hodnoty jsou napevno a neukladaji se do EEPROM
    kal_svetlo[15][0] = 0;                               //  [15]   - tohle musi byt minimalni zmeritelna hodnota svetla (nejvetsi svetlo) (0 mag)  (nic mensiho se nesmi namerit)
    kal_svetlo[15][1] = 0;
    kal_svetlo[16][0] = 24000;                           //  [16]   - tohle musi byt maximalni zmeritelna hodnota svetla (nejvetsi tma) (24,000 mag)(nic vetsiho se nesmi namerit)
    kal_svetlo[16][1] = 24000;

    test_korekcni_tabulky_svetla();                      // zkontroluje spravne zadane udaje



    // nacteni kalibracnich dat pro teplotu z EEPROM
    for (byte i = 0 ; i < 7 ; i++)
      {
        kal_teplota[i][0] = EEPROM_read_int((i*4) + 20);
        kal_teplota[i][1] = EEPROM_read_int((i*4) + 22);
      }
    // nasledujici 2 kalibracni hodnoty jsou napevno a neukladaji se do EEPROM
    kal_teplota[7][0] = 0;                               //  [7]   - tohle musi byt minimalni zmeritelna hodnota teploty (-50'C)(nic mensiho se nesmi namerit)
    kal_teplota[7][1] = 0;
    kal_teplota[8][0] = 10000;                           //  [8]   - tohle musi byt maximalni zmeritelna hodnota teploty (+50'C) (nic vetsiho se nesmi namerit)
    kal_teplota[8][1] = 10000;

    test_korekcni_tabulky_teploty();                     // zkontroluje spravne zadane udaje

    // vytvoreni unikatniho identifikacni cislo procesoru (cislo waferu, x-y pozice cipu na waferu, vyrobni sarze)
    long32_to_HEX(ID1, 0);
    long32_to_HEX(ID2, 10);
    long32_to_HEX(ID3, 21);

    lux_kal_a = ((int16_t)EEPROM_read_int(583)) / 100.0; // nacteni kalibracnich konstant pro luxmetr (v EEPROM mohou byt ulozena i zaporna cisla -32768 az +32767)
    lux_kal_b = ((int16_t)EEPROM_read_int(585)) / 100.0;
    lux_kal_c = ((int16_t)EEPROM_read_int(587)) / 1000000.0;


    prepni_cidlo_svetla(1);                              // v pripade pouziti rozsirujici desky pro vice cidel svetla se prepne na to prvni.
                                                         //   kdyz deska neexistuje, zustane jen jedno cidlo na zakladni desce

//       ---- zacatek bloku R30s --


















//       ---- konec bloku R30s --
    
    if (automat > 0)                                     // kdyz je nastavene automaticke snimani
      {
        posledni_autospusteni = millis();
        zobraz_text(1);                                  // "Auto "
        delay(200);

//        mereni(6);                                     // odkomentovanim by se prvni mereni provedlo hned po zapnuti napajeni
      }  

#ifndef omezeni_funkci_4                                 // pokud je logovani povoleno (neni omezena funkce 4) ...
    SD_log(999 , 1);                                     // ... uklada se do zvlastniho logovaciho souboru na SD karte datum a cas normalniho zapnuti napajeni (kod/adresa 999)
#endif   
  
  }
//======================================================================================================================



//======================================================================================================================
void loop(void)
  {

    test_baterie();

#ifndef omezeni_funkci_1                                 // tuto funkci je mozne kvuli uspore prostoru v PROGMEM vypnout
    if(digitalRead(pin_TEST_HW) == HIGH)                 // jen testy A/D prevodniku pro vyhodnoceni stavu baterie - bez pripojene komunikace
      {
        zobraz_text(34);                                 // "rEF_U"
        ADC_CR2 = ADC_CR2 | 0b1;                                  // zapnuti A/D prevodniku
        delay(3800);        
        zobraz_cislo(analogRead(pin_refu),0);
        delay(2500);

        zobraz_text(35);                                 // "U_bAt"
        delay(3800);        
        zobraz_cislo(analogRead(pin_AD_BAT),0);
        delay(2500);    
      }
#endif

#ifndef omezeni_funkci_4                                 // tuto funkci je mozne kvuli uspore prostoru v PROGMEM vypnout
    if (digitalRead(pin_EXT_ZAP) != predchozi_data_RDY)  // kdyz dojde k prepnuti stavu pinu 'pin_EXT_ZAP' (zapnuti / vypnuti GPS)
      {
        predchozi_data_RDY = digitalRead(pin_EXT_ZAP);   // zaloguje se tento stav
        SD_log(997,predchozi_data_RDY);
      }
#endif


    interrupty();                                        // odskoceni na test, jestli neni nejaky pozadavek o komunikaci, nebo jestli neni cas spustit automaticke mereni
    auto_exit_menu = false;                              // v hlavni smycce se rusi pozadavek na automaticke ukonceni menu

    if (digitalRead(pin_tl_up) == LOW)                   // zaznam hodnoty s tlacitkem "NAHORU"
      {
        delay(20);
        mereni(1);                                       // parametr je znacka tlacitka, ktera se uklada do zaznamu
        while (digitalRead(pin_tl_up) == LOW) delay(20); // cekani na uvolneni tlacitka
        delay (20);                                      // odruseni zakmitu pri uvolneni tlacitka "NAHORU"
      }


    if (digitalRead(pin_tl_dn) == LOW)                   // zaznam hodnoty s tlacitkem "DOLU"
      {
        delay(20);
        mereni(2);                                       // parametr je znacka tlacitka, ktera se uklada do zaznamu
        while (digitalRead(pin_tl_dn) == LOW) delay(20); // cekani na uvolneni tlacitka
        delay (20);                                      // odruseni zakmitu pri uvolneni tlacitka "NAHORU"
      }


    if (digitalRead(pin_tl_ok) == LOW)                   // stisk tlacitka OK (tady se rozlisuje kratky a dlouhy stisk)
      {
        delay(50);                                       // odruseni zakmitu pri stisku tlacitka
        stisk_time = millis();
        stisk = true;
        unsigned long trvani = 0;
        while (digitalRead(pin_tl_ok) == LOW)
          {
            delay(20);                                   // cekani na uvolneni tlacitka
            trvani = millis() - stisk_time;
            if (trvani >= 500)                           // dlouhy stisk = vstup do menu
              {
                
#ifdef signal_LED
                LED_R_onoff(true);
                menu_LED();                              // vstup do LED menu (jen zakladni polozky, ktere jsou signalizovany blikanim LED )
#else
                menu_1();                                // vstup do menu se zobrazenim polozek na nekolika 7-segmentovych displejich
#endif


              }
            if (digitalRead(pin_tl_ok) == HIGH) delay(50); // kdyz se behem drzeni tlacitka OK objevi kratky vypadek kontaktu, tak se odignoruje        
          }
        if (trvani < 500 and trvani > 100)               // kratky stisk = mereni (stisk ale musi trvat alespon 0,1 sekundy)
          {
            mereni(3);                                   // pro tlacitko OK se provadi mereni az pri uvolneni
          }       
    
        stisk=false;
      }

    msleep(100);
  
  }
//======================================================================================================================



//------------------------------------------
// systemove informace
void sys_info(void)
  {
    unsigned int pomprom;
    
    Serial.print("SW ver.: ");                                 //             verze software
    Serial.println(verzeSW);                                   //             (datum nebo jiny text nadefinovany na zacatku programu)
    Serial.println(' ');                                       //             prazdna radka

    
    pomprom = EEPROM_read(8);
    Serial.print(lng111);                                      //             Serial.print("Casova zona: ");
    if (pomprom == 0)Serial.print(char_zima);                  //             Serial.print("SEC ");
    else             Serial.print(char_leto);                  //             Serial.print("SELC");

    
    Serial.print(" (UTC ");
    if (pomprom == 0)
      {
        if (zimni_posun >= 0)  Serial.print('+');
        Serial.print(zimni_posun);
      }
    else
      {
        if (letni_posun >= 0)  Serial.print('+');
        Serial.print(letni_posun);
      }
    Serial.println(')');


    pomprom = EEPROM_read(9);
    Serial.println(lng112);                                    //             Serial.println("\nOddelovace:");
    
    Serial.print(lng113);                                      //             Serial.print(" Polozky v souboru: ");
    if ((pomprom & 0b11) ==     0 )  Serial.println(lng078);   //             Serial.println("mezera");
    if ((pomprom & 0b11) ==     1 )  Serial.println(lng079);   //             Serial.println("carka");
    if ((pomprom & 0b11) ==     2 )  Serial.println(lng080);   //             Serial.println("strednik");
    if ((pomprom & 0b11) ==     3 )  Serial.println(lng081);   //             Serial.println("tabulator");

    Serial.print(lng114);                                      //             Serial.print(" Desetinny oddelovac: ");
    if ((pomprom & 0b100) ==    4 )  Serial.println(lng079);   //             Serial.println("carka");
    else                             Serial.println(lng083);   //             Serial.println("tecka");

    if ((pomprom & 0b1000) ==   8 )  Serial.println(lng115);   //             Serial.println(" Uzavirat polozky do uvozovek");
    else                             Serial.println(lng116);   //             Serial.println(" Neuzavirat polozky do uvozovek");

    if ((pomprom & 0b10000) == 16 )  Serial.println(lng117);   //             Serial.println(" Do souboru vkladat hlavicky");
    else                             Serial.println(lng118);   //             Serial.println(" Hlavicky do souboru nevkladat");

    Serial.print(lng119);                                      //             Serial.println("\nAutomat:");        
    pomprom = EEPROM_read(10);
    if (pomprom == 0)                Serial.println(lng120);   //             Serial.println(" Bez automatickeho mereni");
    else
      {
                                     Serial.print(lng121);     //             Serial.print("  Spoustet mereni kazdych ");
                                     Serial.print(pomprom);
                                     Serial.println(lng122);   //             Serial.println(" minut");
      }

    pomprom = EEPROM_read(11);
    Serial.print(lng123);                                      //             Serial.print("\nPrumerovat ");
    Serial.print(pomprom);
    Serial.println(lng124);                                    //             Serial.println(" mereni plosneho jasu");

#ifdef pouzit_modbus
    Serial.println(lng272);                                    //             Serial.println("MODBUS: zapnuty");
#else
    pomprom = (EEPROM_read(15) & 0b00000001);                  //             informace o CRC je v bitu 0
    
    if (pomprom == 0)  Serial.println(lng125);                 //             Serial.println("Kontrolni bajt se pri komunikaci pres RS485 nevyhodnocuje.");
    else               Serial.println(lng126);                 //             Serial.println("Pri komunikaci pres RS485 je nutne odesilat spravny kontrolni bajt.");
#endif

    pomprom = EEPROM_read(16);
    Serial.print(lng127);                                      //             Serial.print("Rozhodovaci rozdil pro stabilni/nestabilni jas je ");
    Serial.print(pomprom/10.0,1);
    Serial.println(" %");

    pomprom = EEPROM_read(19);
    if (pomprom > 0)
      {
        Serial.print(lng128);                                  //             Serial.print("Do EEPROM se ukladaji zaznamy s plosnym jasem nad (tmavsi nez) ");
        Serial.print(pomprom/10.0);
        Serial.println(" mag/arcsec2");
      }
    else
      {
        Serial.println(lng207);                                //             Serial.print("Do EEPROM se ukladaji vsechny zaznamy");        
      }
   
    Serial.print(lng129);                                      //             Serial.print("Unikatni identifikace: ");
    Serial.print(" 0x");
    Serial.println(ID_char);

    pomprom = (0b00001111 & EEPROM_read(14));
    Serial.print(lng019);                                      //             Serial.print("SLAVE adresa: ");
    Serial.println(pomprom);

    if (mute_tone_bit == true) Serial.println(lng224);         //             Serial.println("Pipani zakazano");
    else                       Serial.println(lng223);         //             Serial.println("Pipani povoleno");

    Serial.println(lng130);                                    //             Serial.println("\nHardware:");

    Serial.println(lng131);                                    //             Serial.println(" Procesor: STM32F103CB (BluePill) - 128kB progMEM");    
    Serial.println(lng132);                                    //             Serial.println(" Cidlo jasu: TSL2591");    
#ifdef senzor_DHT11
    Serial.println(lng133);                                    //             Serial.println(" Cidlo vlhkosti a teploty: DHT11");
#endif
#ifdef senzor_DHT22
    Serial.println(lng134);                                    //             Serial.println(" Cidlo vlhkosti a teploty: DHT22");
#endif
#ifdef senzor_BME
    Serial.println(lng135);                                    //             Serial.println(" Cidlo vlhkosti, teploty a tlaku: BME280");
#endif
#ifdef modul_LSM303DLHC
    Serial.println(lng265);                                    //             Serial.println(" Kompas / naklon LSM303DLHC");
#endif


    Serial.print(lng136);                                      //             Serial.print(" SD karta: ");
#ifdef pouzivat_SD_kartu
    if (digitalRead(pin_karta_IN) == LOW) Serial.println(lng137);//           Serial.println("zasunuta");
    else                                  Serial.println(lng138);//           Serial.println("vysunuta");
#else
    Serial.println(lng139);                                    //             Serial.println("nepouziva se");
#endif
    
#ifdef displej_TM1637_5
    Serial.println(lng140);                                    //             Serial.println(" Displej: 5x7 segmentu (TM1637)");
#endif

#ifdef signal_LED
    Serial.println(lng141);                                    //             Serial.println(" Pouze signalizacni LED");
#endif

    if (pocet_cidel_svetla > 1)                                // kdyz neni nastavena expanzni deska pro cidla svetla, nema cenu vypisovat hlaseni o dostupnych vstupech
      {
        Serial.print(lng142);                                  //             Serial.print(" Pocet vstupu rozsirujici desky: ");
        Serial.println(pocet_cidel_svetla);
        Serial.print(lng143);                                  //             Serial.print(" Aktualne aktivni cidlo jasu:    ");
        Serial.println(index_cidla_svetla);
      }


    Serial.println(lng144);                                    //             Serial.println("\nEEPROM:");
    Serial.print(lng145);                                      //             Serial.print(" Zacatek oblasti pro data: ");
    dopln_mezery(min_EEPROM,6);
    Serial.println(min_EEPROM);
    
    Serial.print(lng146);                                      //             Serial.print(" Konec oblasti pro data:   ");
    dopln_mezery(max_zaznam_EEPROM + delka_zaznamu - 1 ,6);
    Serial.println(max_zaznam_EEPROM + delka_zaznamu - 1);

    Serial.print(lng147);                                      //             Serial.print(" Pristi zaznam se ulozi na adresu: ");
    Serial.println(adr_dalsiho_zaznamu);


    Serial.print(lng148);                                      //             Serial.print(" Jeden zaznam v EEPROM je dlouhy: ");
    Serial.print(delka_zaznamu);
    Serial.println(lng149);                                    //             Serial.println (" bajtu");

    Serial.println(lng150);                                    //             Serial.println(" obsahuje tyto polozky: ");
    Serial.println(lng151);                                    //             Serial.println("  - plosny jas           ANO");

    Serial.print(lng152);                                      //             Serial.print  ("  - teplota              ");
#ifdef ukladat_teplotu
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng155);                                      //             Serial.print  ("  - tlak                 ");
#ifdef ukladat_tlak
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng156);                                      //             Serial.print  ("  - vlhkost              ");
#ifdef ukladat_vlhkost
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng157);                                      //             Serial.print  ("  - infra slozka svetla  ");
#ifdef ukladat_infra
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng158);                                      //             Serial.print  ("  - full svetlo          ");
#ifdef ukladat_full
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng159);                                      //             Serial.print  ("  - ctrl registr TSL2591 ");        
#ifdef ukladat_contr_reg
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng206);                                      //             Serial.print  ("  - GPS souradnice       ");        
#ifdef ukladat_GPS
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng228);                                      //             Serial.print  ("  - aktualni naklon      ");        
#ifdef ukladat_naklon
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng264);                                      //             Serial.print  ("  - azimut               ");        
#ifdef ukladat_azimut
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif


    Serial.print(lng236);                                      //             Serial.print  ("  - poloha Slun. a Mes.  ");        
#ifdef ukladat_astro
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng237);                                      //             Serial.print("Aktualni souradnice pro astro vypocty (delka / sirka): ")
    Serial.print(GeoLon,1);
    Serial.print(" / ");
    Serial.println(GeoLat,1);

    if (ctdwn_end > 0)                                         //             je aktivni odpocet  
      {
        unsigned long pomprom_ctdwn = rtc_korekce();
        int odpocet_H = (ctdwn_end - pomprom_ctdwn) / 3600;    //             prepocet rozdilu na cele hodiny
        int odpocet_M = ((ctdwn_end - pomprom_ctdwn) - (odpocet_H * 3600UL)) /60;        //   a minuty

        Serial.print(lng247);                                  //            Serial.print("Aktivni odpocet: ")
        Serial.print('-');
        Serial.print(odpocet_H);
        Serial.print(':');
        if (odpocet_M < 10)   Serial.print('0');
        Serial.print(odpocet_M);
        Serial.print("  ");
        sek1970_datcas(ctdwn_end, true);                       //            tenhle podporogram nastavuje promenne "LOC_xxx" na hodnotu casu alarmu
        zobraz_RTC(false);                                     //               proto se hned po zobrazeni casu alarmu musi promenne "LOC_xxx" zase aktualizovat na hodnoty z RTC


      }
    
    Serial.println(lng160);                                    //             Serial.println("\nKalibrace cidla teploty");
    Serial.println(lng161);                                    //             Serial.println("index ; merena  ; skutecna");
    Serial.println("-----------------------------");
    for (byte i = 0 ; i < 7 ; i++)
      {
        pomprom = EEPROM_read_int((i*4)+20);
        Serial.print(i+1);
        Serial.print(" ; ");
        dopln_mezery(citelna_teplota(pomprom),5);
        Serial.print( citelna_teplota(pomprom),2);             // (citelna merena teplota)
        Serial.print(" 'C ; ");

        pomprom = EEPROM_read_int((i*4)+22);
        dopln_mezery(citelna_teplota(pomprom),5);
        Serial.print( citelna_teplota(pomprom),2);             // (citelna zobrazovana teplota)
        Serial.println(" 'C");        
      }


    Serial.print(lng162);                                      //             Serial.print("\nKalibrace cidla plosneho jasu [");
    Serial.print(index_cidla_svetla);
    Serial.println("]");
    Serial.println(lng163);                                    //             Serial.println("index ; merene ; skutecne");
    Serial.println("--------------------------");
    unsigned int posun = ((index_cidla_svetla-1) * 60) + 50; 
    for (byte i = 0 ; i < 15 ; i++)
      {
        if( i < 9)   Serial.print(' ');                        // jen zarovnani cisel polozek
        Serial.print(i+1);
        Serial.print(" ; ");
        pomprom = EEPROM_read_int(posun + (i*4));
        dopln_mezery(citelne_svetlo(pomprom),5);
        Serial.print(citelne_svetlo(pomprom),3);
        Serial.print(" ; ");
        pomprom = EEPROM_read_int(posun + (i*4) + 2);
        dopln_mezery(citelne_svetlo(pomprom),5);
        Serial.println(citelne_svetlo(pomprom),3);
      }    
  
  }
//----------------------------------------------



//---------------------------------------------------------------------
// formatovani vypisu kalibracnich konstant
//   cislo se hned na zacatku oseka o desetinne casti
//  'pocet_mist' udava, na jak dlouhy retezec se doplni cela cast cisla
//   priklady parametru:    15.354 , 4     prida pred retezec 2 mezery
//                          1.078  , 4     prida pred retezec 3 mezery
//                         -24.11  , 3     neprida zadnou mezeru
void dopln_mezery(int cislo, byte pocet_mist)
  {
    byte delka_cisla = 5;
    if (cislo <     0)   delka_cisla ++;                       // zaporne cislo pridava jeden znak pro znamenko
    cislo = abs(cislo);
    if (cislo < 10000)   delka_cisla --;
    if (cislo <  1000)   delka_cisla --;
    if (cislo <   100)   delka_cisla --;
    if (cislo <    10)   delka_cisla --;

    while (delka_cisla < pocet_mist)
      {
        delka_cisla ++;
        Serial.print(' ');
      }
    
  }
//----------------------------------------------



//----------------------------------------------
void default_kalibrace_svetla(void)
  {

    for (byte i = 1; i < 8 ; i++)
      {
        // v EEPROM se nachazi 7 bloku kalibracnich dat (pro 7 cidel na rozsirujici desce), kazdy blok ma 15 bodu, kazdy bod ma 2 + 2 bajty (merena a skutecna hodnota) = 60 bajtu na blok
        //               bloky zacinaji na adrese 50 (za kalibracni tabulkou teplot)
        // Nasledujici zapis nulovych hodnot by se dal udelat jednoduchou smyckou, ale pro pripad, ze by se ukazalo, ze je lepsi zapisovat nejake jine hodnoty jsem to nechal oddelene
        
        unsigned int posun = ((i-1) * 60) + 50;   
        EEPROM_write_int( 0 + posun   ,     0);                //  1. kalibracni bod pro svetlo -  index[0]
        EEPROM_write_int( 2 + posun   ,     0);
    
        EEPROM_write_int( 4 + posun   ,     0);                //  2. kalibracni bod pro svetlo -  index[1]
        EEPROM_write_int( 6 + posun   ,     0);
    
        EEPROM_write_int( 8 + posun   ,     0);                //  3. kalibracni bod pro svetlo -  index[2]
        EEPROM_write_int(10 + posun   ,     0);
    
        EEPROM_write_int(12 + posun   ,     0);                //  4. kalibracni bod pro svetlo -  index[3]
        EEPROM_write_int(14 + posun   ,     0);   
    
        EEPROM_write_int(16 + posun   ,     0);                //  5. kalibracni bod pro svetlo -  index[4]
        EEPROM_write_int(18 + posun   ,     0);
    
        EEPROM_write_int(20 + posun   ,     0);                //  6. kalibracni bod pro svetlo -  index[5]
        EEPROM_write_int(22 + posun   ,     0);
    
        EEPROM_write_int(24 + posun   ,     0);                //  7. kalibracni bod pro svetlo -  index[6]
        EEPROM_write_int(26 + posun   ,     0);
    
        EEPROM_write_int(28 + posun   ,     0);                //  8. kalibracni bod pro svetlo -  index[7]
        EEPROM_write_int(30 + posun   ,     0);
    
        EEPROM_write_int(32 + posun   ,     0);                //  9. kalibracni bod pro svetlo -  index[8]
        EEPROM_write_int(34 + posun   ,     0);
    
        EEPROM_write_int(36 + posun   ,     0);                // 10. kalibracni bod pro svetlo -  index[9]
        EEPROM_write_int(38 + posun   ,     0);
    
        EEPROM_write_int(40 + posun   ,     0);                // 11. kalibracni bod pro svetlo -  index[10]
        EEPROM_write_int(42 + posun   ,     0);
    
        EEPROM_write_int(44 + posun   ,     0);                // 12. kalibracni bod pro svetlo -  index[11]
        EEPROM_write_int(46 + posun   ,     0);
    
        EEPROM_write_int(48 + posun   ,     0);                // 13. kalibracni bod pro svetlo -  index[12]
        EEPROM_write_int(50 + posun   ,     0);
        
        EEPROM_write_int(52 + posun   ,     0);                // 14. kalibracni bod pro svetlo -  index[13]
        EEPROM_write_int(54 + posun   ,     0);
    
        EEPROM_write_int(56 + posun   ,     0);                // 15. kalibracni bod pro svetlo -  index[14]
        EEPROM_write_int(58 + posun   ,     0);
        
      }
  }
//----------------------------------------------



//----------------------------------------------
void default_kalibrace_teploty(void)
  {
    // Nasledujici zapis hodnot 0 by se dal udelat jednoduchou smyckou, ale pro pripad, ze by se ukazalo, ze je lepsi zapisovat nejake jine hodnoty jsem to nechal oddelene
    EEPROM_write_int(20   ,  0);                               // 1. kalibracni bod pro teplotu -  index[0]   
    EEPROM_write_int(22   ,  0);

    EEPROM_write_int(24   ,  0);                               // 2. kalibracni bod pro teplotu -  index[1]   
    EEPROM_write_int(26   ,  0);

    EEPROM_write_int(28   ,  0);                               // 3. kalibracni bod pro teplotu -  index[2]   
    EEPROM_write_int(30   ,  0);

    EEPROM_write_int(32   ,  0);                               // 4. kalibracni bod pro teplotu -  index[3]   
    EEPROM_write_int(34   ,  0);

    EEPROM_write_int(36   ,  0);                               // 5. kalibracni bod pro teplotu -  index[4]   
    EEPROM_write_int(38   ,  0);

    EEPROM_write_int(40   ,  0);                               // 6. kalibracni bod pro teplotu -  index[5]   
    EEPROM_write_int(42   ,  0); 

    EEPROM_write_int(44   ,  0);                               // 7. kalibracni bod pro teplotu -  index[6]   
    EEPROM_write_int(46   ,  0); 
  }

void default_parametry(void)
  {
    EEPROM_write_long(    0   ,            0);                 // cas posledniho serizeni = 1.1.1970 0:00:00
    EEPROM_write_long(    4   , 0x7FFFFFFFUL);                 // sekunda se pricita jednou za 68 let
    EEPROM_write(         8   ,            0);                 // SEC
    EEPROM_write(         9   ,   0b00010110);                 // des. oddelovac carka, oddelovac polozek strednik, hlavicku vkladat, nezavirat do uvozovek
    EEPROM_write(        10   ,            0);                 // automaticke spousteni vypnute
    EEPROM_write(        11   ,            3);                 // 3x prumerovani
//     EEPROM_write_int(    12   ,            0);              // smazat unikatni identifikaci (po resetu se vytvori nova)
    EEPROM_write(        14   ,            1);                 // komunikacni rychlost 9600; SLAVE adresa = 1
    EEPROM_write(        15   ,            1);                 // testuje kontrolni bajt pri komunikaci pres RS485; tlacitko TS je odemknute, RTC_set.csv zakazan
    EEPROM_write(        16   ,           20);                 // nestabilni jas je hlaseny pri rozdilu dvou sousednich mereni nad 0.2 %
    EEPROM_write(        17   ,            1);                 // pocet cidel na expanzni desce
    EEPROM_write(        18   ,            0);                 // zachytny bod hledani (po kazdem 100. zaznamu se aktualizuje kvuli rychlejsimu hledani volneho mista v EEPROM)
    EEPROM_write(        19   ,            0);                 // hranice plosneho jasu pro ukladani do EEPROM a na kartu v desetinach mag/arcsec2) (0 az 25,5 mag/arcsec2)
    EEPROM_write_int(   eeaddr_GEO_lat   ,          500);                 // domaci zemepisna sirka pro astro vypocty na 50.0 stupnu
    EEPROM_write_int(   eeaddr_GEO_lon   ,          150);                 // domaci zemepisna delka pro astro vypocty na 15.0 stupnu
    EEPROM_write(       489   ,          101);                 // posun casu proti UTC v zime = +1 hodina (SEC)
    EEPROM_write(       490   ,          102);                 // posun casu proti UTC v lete = +2 hodiny (SELC)
    EEPROM_write_long(  491   , 0xFFFFFFFFUL);                 // defaultne jsou vsechny polozky v menu povolene
    EEPROM_write_long(  495   ,            0);                 // funkce odpocet (budik) je vypnuta
    EEPROM_write     (  575   , ' ');                          // 4 ASCII znaky pro 'zimni' casovou zonu
    EEPROM_write     (  576   , 'S');                          // 
    EEPROM_write     (  577   , 'E');                          // 
    EEPROM_write     (  578   , 'C');                          // 
    EEPROM_write     (  579   , 'S');                          // 4 ASCII znaky pro letni casovou zonu
    EEPROM_write     (  580   , 'E');                          // 
    EEPROM_write     (  581   , 'L');                          // 
    EEPROM_write     (  582   , 'C');                          // 
    EEPROM_write_int(   583   ,          250);                 // parametr 'a' pro kalibraci luxmetru (*0.01)   (s ulozenym cislem 250 se bude pocitat jako s 2.5)
    EEPROM_write_int(   585   ,            0);                 // parametr 'b' pro kalibraci luxmetru (*0.01)  zaporna cisla se ukladaji jako binarni doplnek (-10.55 = 64481)
    EEPROM_write_int(   587   ,            0);                 // parametr 'c' pro kalibraci luxmetru (*0.000 000 1) ulozene cislo 16 by se pocitalo jako 0.000 001 6

#ifdef modul_LSM303DLHC
    default_naklonomer();                                      // prednastaveni defaultnich hodnot pro naklonomer
    default_kompas();                                          // prednastaveni defaultnich hodnot pro kompas
#endif

    default_stanoviste();                                      // prednastaveni popisku pro 5 pozorovacich stanovist 

     
  }
//----------------------------------------------



//----------------------------------------------
void default_stanoviste(void)
  {
    for (byte i = 1; i < 6 ; i ++)
      {
        for (byte j = 0; j < 10 ; j ++)
          {
            EEPROM_write(485 + (i * 15) + j , poz_st_txt[i].charAt(j));            
          }

        for (byte j = 0; j < 5 ; j ++)
          {
            EEPROM_write(495 + (i * 15) + j , poz_st_gr[i][j]);            
          }
      } 
  }
//----------------------------------------------



//----------------------------------------------
void chyba(byte kod_chyby)
  {
#ifndef omezeni_funkci_4                                 // pokud je logovani povoleno (neni omezena funkce 4) ...
    SD_log(998 , kod_chyby);                             // ... uklada se do zvlastniho logovaciho souboru na SD karte datum a cas vzniku chyby (kod/adresa = 998, data = cislo chyby)
#endif   

    
#ifdef displej_TM1637_5    
    switch(kod_chyby)
      {
        case 1:                                                    // chyba I2C komunikace (cidlo svetla, EEPROM, expander, naklonomer, kompas)
          zobraz_text(40);                                         //   Err-i
          Wire.begin();                                            // pokus o restart I2C
          break;

        case 2:                                                    // chyba SD karty
          zobraz_text(37);                                         //   Sd-Er
          break;

        case 3:                                                    // neni zasunuta SD karta
          zobraz_text(45);                                         //   no-5d  (No-SD)
          break;

        case 4:                                                    // Spatny datum v RTC
          zobraz_text(61);                                         //   Err-r
          break;

        case 5:                                                    // Chyba v kalibracni tabulce svetla
          zobraz_text(30);                                         //   Err-S
          break;

        case 6:                                                    // Chyba v kalibracni tabulce teploty
          zobraz_text(31);                                         //   Err-t
          break;

        case 7:                                                    // blokovane bocni tlacitko
          zobraz_text(36);                                         //   -Loc-
          break;

        case 8:                                                    // chyba naklonomeru
          zobraz_text(55);                                         //   Err-n
          break;

        case 9:                                                    // chyba kalibrace kompasu
          zobraz_text(60);                                         //   Err-C
          break;

        case 10:                                                   // chyba formatovani
          zobraz_text(63);                                         //   Err-F
          break;

        default:
          break;
      }

    if (pocet_bzuku > 0)                                           // od zapnuti napajeni je pocet pipnuti pri chybach omezen na 5 (to je proto, aby se pri opakovanych chybach komunikace nebzukalo nepretrzite)
      {
        pocet_bzuku --;
        for (byte bz = 0; bz < 3 ; bz++ )                          // pri chybe se jeste 3x rychle zapipa
          {
            tone_X(pin_bzuk,600, 10);                              // pokud ale neni pipani zakazane prikazem "@P"
            delay(100);
          }    
      }
    
    delay(500);
    
    zobraz_text(28);                                               // po pul sekunde se zhasne displej
        
    
#endif

#ifdef signal_LED    
    LED_B_onoff(true);                                             // rozsviti modrou LED
    delay(500);
#endif
  }
//----------------------------------------------



//----------------------------------------------
// prepinani cidel svetla
//  kdyz je index zadany, prepne se na konkretni cislo
void prepni_cidlo_svetla(byte index)
  {
    if (index <= pocet_cidel_svetla and index > 0)
      {
        index_cidla_svetla = index;
        
        // nacist novou kalibracni tabulku z EEPROM podle aktualne prepnuteho cidla svetla
        unsigned int posun = ((index_cidla_svetla-1) * 60) + 50;   // podle aktualniho indexu cidla svetla se zvoli spravna oblast v EEPROM pro kalibracni tabulku
        for (byte i = 0 ; i < 15 ; i++)
          {
            kal_svetlo[i][0] = EEPROM_read_int(posun   +   (i*4));
            kal_svetlo[i][1] = EEPROM_read_int(posun + 2 + (i*4));
          }
#ifndef omezeni_funkci_5          
        expa_switch(index_cidla_svetla);                           // prepnout napajeni na prislusne cidlo
#endif
      
      }   // pri spatne zadane hodnote se nic nedeje
  }
//----------------------------------------------



//----------------------------------------------
// kdyz parametr zadany neni, prepne se na nasledujici cidlo
void prepni_cidlo_svetla(void)
  {
    index_cidla_svetla ++;
    if (index_cidla_svetla > pocet_cidel_svetla) index_cidla_svetla = 1;
    prepni_cidlo_svetla(index_cidla_svetla);    
  }
//----------------------------------------------



//----------------------------------------------
// pri dlouhych operacich (nebo ve smyckach, ktere bezi mimo hlavni loop) se pravidelne kontroluje, jestli doslo k nejakemu pozadavku o komunikaci,
// nebo jestli uz neni cas provest automaticke spusteni mereni
void interrupty(void)
  {

#ifdef pouzit_modbus    
    test_485_mod();
#else
    test_485_kom();                                                   // testovani, jsetli neprislo neco po seriove lince RS485
#endif

    
        
    test_ser_kom();                                                   // testovani, jsetli neprislo neco po USB lince

    if (digitalRead(pin_TS) == LOW and blokuj_TS == false and TS_lock == false) // pri zkratu pinu SCK a GND na bocnim konektoru desky BluePill + se zaznamena casove razitko
      {
        blokuj_TS = true;                                             // zablokovani dalsiho zapisu casove znacky az do uvolneni tlacitka
        casove_razitko();
      }

    if (digitalRead(pin_TS) == HIGH)
      {
        blokuj_TS = false;                                            // odblokovani dalsiho zapisu casove znacky po uvolneni tlacitka
      }

    if (automat > 0)                                                  // kdyz je nastavene automaticke snimani
      {
        if (millis() - posledni_autospusteni > (automat * 60000UL))   // automat udava cas v minutach - to je (60 * 1000ms)
          {
            posledni_autospusteni = millis();  
            mereni(6);                                                // spusteni automatickeho mereni          
          }
      }      

//       ---- zacatek bloku R30s --


//       ---- konec bloku R30s --

    if (ctdwn_end > 0)                                                // konec odpoctu se testuje jen kdyz je nejaky nastaveny
      {
        if (rtc_korekce() > ctdwn_end)  alarm();                      // aktualni cas je vetsi nez cilovy cas alarmu = spusti se pulsekundove bzukani
      }

  
  }
//----------------------------------------------



//----------------------------------------------
// Prepise jen jeden bit v kazdem zaznamu, takze je prostor oznaceny jako prazdny.
// Data jsou ale az do dalsiho prepsani citelna (ve vypisech ze poznaji podle toho, ze misto napisu " SEC" nebo "SELC" maji zobrazeno " _EC", nebo "_ELC")
void soft_format_EEPROM(void)
  {
    zobraz_text(28);                                                  // smaze displej
    Serial.println(lng164);                                           // Serial.println("SOFT format EEPROM ...");    // bargraf do serioveho terminalu
    unsigned int pocitadlo = 0;

    pomlcky((((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100));
//    Serial.print('|');
//    for (byte i = 0; i< (((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) ; i++)
//      {
//        Serial.print('-');
//      }
//    Serial.println('|');

    Serial.print('|');

    zobraz_cislo((((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) - (min_EEPROM / delka_zaznamu  / 100) , 7);        // vystup odpocitavani na displej podle delky zaznamu se zacina na 34 az 160 (pro 3400 az 16000 zaznamu) 

    dokoncene_mereni = 0;                                             // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres RS485    
    for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
      {
#ifdef pouzit_modbus
        test_485_mod();                                               // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
        test_485_kom();                                               // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif
        pocitadlo ++;
        if ((pocitadlo % 100) == 0)
          {
            Serial.print('#');
            zobraz_cislo((((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) - (adresa / delka_zaznamu  / 100) , 7);        // vystup odpocitavani na displej podle delky zaznamu se zacina na 34 az 160 (pro 3400 az 16000 zaznamu) 
          }
        byte pomprom = EEPROM_read(adresa);
        if ((pomprom & 0b10000000) > 0)                               // pokud je v EEPROM na zacatku bloku nastaveny bit7 na '1', znamena to obsazeny blok a tato '1' se smaze
          {
            delay(3);
            EEPROM_write(adresa , (pomprom & 0b01111111));            // bit7 se nastavi na '0' 
          }

         if (digitalRead(pin_tl_ok) == LOW)                           // moznost predcasneho ukonceni tlacitkem OK
           {
              delay(50);
              unsigned long exit_znacka = millis();
              while (digitalRead(pin_tl_ok) == LOW)                   // ceka se na uvolneni tlacitka OK
                {
                  graf_natazeni(millis() - exit_znacka , 113);        // zaroven se na displeji zobrazuje pomlckovy bargraf
                  delay(20);
                }
              if (millis() - exit_znacka > 2000)                      // kdyz stisk trval alespon 2 sekundy, smycka for se prerusi 
                {
                  chyba(10);                                          // "Err-F"
                  delay(1000);
                  break;              
                }
              else zobraz_cislo((((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) - (adresa / delka_zaznamu  / 100) , 7);
           }
      
      }
    EEPROM_write(18 ,0);                                              // zachytny bod pro pristi hledani volneho mista v EEPROM se nastavi na zacatek (adresa 'min_EEPROM')
    
    dokoncene_mereni = 2;                                             // po skonceni formatovani se odblokuje zablokovane spousteni dalsiho mereni pres RS485

    adr_posl_zaznamu = 0;                                             // nula znamena, ze jeste neni zadny zaznam. Pouziva se pri blokovani vypisu okamzite po formatu.
    
    Serial.println('|');
    Serial.println(lng166);                                           //             Serial.println("EEPROM byla zformatovana.");

    delay(1000);
    SOFT_RESET();
    
  }
//----------------------------------------------



//----------------------------------------------
// smaze uplne vsechny datove bunky v EEPROM
void hard_format_EEPROM(void)
  {
    zobraz_text(28);                                                  // smaze displej
    Serial.println(lng165);                                           //             Serial.println("HARD format EEPROM ...");        // bargraf do serioveho terminalu

    pomlcky(1+((max_EEPROM-min_EEPROM) / 1000));
    
    Serial.print('|');

    zobraz_cislo((max_EEPROM / 1000) , 5);
    dokoncene_mereni = 0;                                             // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni   
    for (unsigned long adresa = min_EEPROM; adresa < max_EEPROM ; adresa ++)
      {
#ifdef pouzit_modbus
         test_485_mod();                                              // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
         test_485_kom();                                              // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif
         if (adresa % 1000 == 0)
           {
             Serial.print('#');
                                                                      // vystup odpocitavani na displej (zacina se od 123 a pri kazdem kilobajtu se odecte 1 ) 
             zobraz_cislo((max_EEPROM / 1000) - ((adresa - min_EEPROM) / 1000) , 5);
           }
         delay(3);
         EEPROM_write(adresa ,0);                                     // smazani bunky
         if (digitalRead(pin_tl_ok) == LOW)                           // moznost predcasneho ukonceni tlacitkem OK
           {
              delay(50);
              unsigned long exit_znacka = millis();
              while (digitalRead(pin_tl_ok) == LOW)                   // ceka se na uvolneni tlacitka OK
                {                
                  graf_natazeni(millis() - exit_znacka , 113);        // zaroven se na displeji zobrazuje pomlckovy bargraf
                  delay(20);
                }
              if (millis() - exit_znacka > 2000)                      // kdyz stisk trval alespon 2 sekundy, smycka for se prerusi 
                {
                  chyba(10);                                          // "Err-F"
                  delay(1000);
                  break;              
                }
              else zobraz_cislo((max_EEPROM / 1000) - ((adresa - min_EEPROM) / 1000) , 5);
           }
      }
    EEPROM_write(18 ,0);                                              // zachytny bod pro pristi hledani volneho mista v EEPROM se nastavi na zacatek (adresa 'min_EEPROM')

    dokoncene_mereni = 2;                                             // po skonceni formatovani se odblokuje zablokovane spousteni dalsiho mereni pres RS485

    adr_posl_zaznamu = 0;                                             // nula znamena, ze jeste neni zadny zaznam. Pouziva se pri blokovani vypisu okamzite po formatu.
    
    Serial.println('|');
    Serial.println(lng166);                                           //             Serial.println("EEPROM byla zformatovana.");
    delay(1000);
    SOFT_RESET();
  
  }
//----------------------------------------------


//----------------------------------------------
#ifndef omezeni_funkci_3                                      // tuto funkci je mozne kvuli uspore prostoru v PROGMEM vypnout
void vypis_LU(void)                                           // vypis posledniho zaznamu ve formatu SQM-LU (USB)
  {
    if (adr_posl_zaznamu > 0)
      {
        priprav_1_zaznam(adr_posl_zaznamu);                   // v promenne 'adr_posl_zaznamu' je ulozena adresa zacatku posledniho zaznamu
        // "r, 06.70m,0000022921Hz,0000000020c,0000000.000s, 039.4C"
        Serial.print("r, ");
        float UHJ = (vystupni_retezec[75]-48) * 10 + vystupni_retezec[76]-48 +  (vystupni_retezec[78]-48) * 0.1 + (vystupni_retezec[79]-48) * 0.01 + (vystupni_retezec[80]-48) * 0.001;
        if (UHJ < 10) Serial.print('0');
        Serial.print (UHJ,2);       
        Serial.print("m,0000022921Hz,0000000020c,0000000.000s,");
        if (vystupni_retezec[151] == '+')   Serial.print(' ');
        else                                Serial.print('-');
        float UHT = (vystupni_retezec[152]-48) * 10 + vystupni_retezec[153]-48 +  (vystupni_retezec[155]-48) * 0.1 + (vystupni_retezec[156]-48) * 0.01;
        if (UHT < 10) Serial.print('0');
        Serial.print (UHT,1);               
        Serial.println('C');
        
      }

  }
#endif  
//----------------------------------------------
void vypis_EEPROM(unsigned int parametr, unsigned int casova_hloubka) // casova_hloubka je pouzita jen pro zadani, jak stare zaznamy se maji vypsat
  {
    boolean prazdno = true;                                           // kdyz nebude ve vypisu ani jeden zaznam, zahlasi se na konci "Zadny ulozeny zaznam"

    if (parametr != 65531)    Serial.println(hlavicka);               // v seriovem terminalu se hlavicka zobrazuje vzdycky (krome funkce LIST z menu)


    switch (parametr)
      {

        case   65534:                                                 // vypis posledniho zaznamu
          {
            if (adr_posl_zaznamu > 0)
              {
                priprav_1_zaznam(adr_posl_zaznamu);                   // v promenne 'adr_posl_zaznamu' je ulozena adresa zacatku posledniho zaznamu
                Serial.println(vystupni_retezec);
                prazdno = false;            
              }
            else
              {
                prazdno = true;            
              }
          }
        break;
        
        
        case  65533:                                                  // vypis vsech zaznamu (zaznamu ktere maji znacku v prvnim bajtu) - nevypisuji se SOFT formatovane zaznamy
          {
            dokoncene_mereni = 0;                                     // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres seriovou linku nebo pres autoomat

            unsigned int pocitadlo_radek = (max_zaznam_EEPROM - min_EEPROM) / delka_zaznamu;
            for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
              {
                if ((EEPROM_read(adresa) & 0b10000000) != 0)          // polozka v EEPROM je obsazena
                  {
                    prazdno = false;
                    priprav_1_zaznam(adresa);
                    Serial.println(vystupni_retezec);
#ifdef pouzit_modbus
                    test_485_mod();                                   // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
                    test_485_kom();                                   // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif
                    if (digitalRead(pin_tl_ok) == LOW)                // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                  }

                pocitadlo_radek --;
                if (pocitadlo_radek % 20 == 0)
                  {
                    zobraz_cislo((pocitadlo_radek / 20) , 6);         // vystup odpocitavani na displej
                  }
              }                     
          }
        dokoncene_mereni = 2;                                         // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni

        break;


        case  65530:                                                  // vypis poslednich zaznamu za obdobi poslednich zadanych hodin
          {
            dokoncene_mereni = 0;                                     // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres RS485
            if (casova_hloubka == 0)     casova_hloubka = 24;         // bez zadane casove hloubky se predpoklada hloubka 24 hodin
            unsigned long historie = rtc_korekce() - (casova_hloubka * 3600UL);    // prevod casu z RTC na mistni cas vcetne autokorekce

            unsigned int pocitadlo_radek = (max_zaznam_EEPROM - min_EEPROM) / delka_zaznamu;            
            zobraz_cislo((pocitadlo_radek / 20) , 6);                 // vystup odpocitavani na displej
            boolean vypis_STOPLAP = false;                            // u vypisu zaznamu stopek typ "STOP" a "LAP" se neuklada do EEPROM aktualni cas, proto se musi vypisovat podle teto znacky 
            for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
              {

                if (EEPROM_read(adresa) >= 127)                       // Vypisuji se jen plne zaznamy. Zaznamy ktere byly smazane SOFT formatem se ignoruji.
                  {
                    if (EEPROM_read_long(adresa+1) >= historie)       // vypisuji se jen zaznamy, ktere maji datum vetsi, nez je rozdil aktualniho casu a zadane casove hloubky
                      {
                        vypis_STOPLAP = true;                         // od prvniho vypisovaneho zaznamu se budou vypisovat i zaznamy "STOP" a "LAP" - nezavisle na hodnote datumu a casu
                        prazdno = false;
                        priprav_1_zaznam(adresa);
                        Serial.println(vystupni_retezec);
#ifdef pouzit_modbus
                        test_485_mod();                               // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
                        test_485_kom();                               // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif    
                        if (digitalRead(pin_tl_ok) == LOW)            // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                          {
                            delay(20);
                            while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                            break;
                          }                      
                      }
                    else                                              // zaznam neodopvida pozadovane hloubce historie
                      {
                        if (vypis_STOPLAP == true)                    // ale pokud je nastavena tato znacka ...
                          {
                            if ((EEPROM_read(adresa) & 0b01000000) > 0)   // ... a zaroven se jedna o specialni pripad zaznamu pro funkci stopky ...
                              {
                                if ((EEPROM_read(adresa + 7) == 2) or (EEPROM_read(adresa + 7) == 3) )   // ... a zaroven se jedna o typ zaznamu "STOP" nebo "LAP"
                                  {
                                    priprav_1_zaznam(adresa);         // tak se zaznam vytiskne bez ohledu na jeho datum a cas
                                    Serial.println(vystupni_retezec);                        
                                  }
                              }
                             
                          }
                      }
                     
                  }

                pocitadlo_radek --;
                if (pocitadlo_radek % 20 == 0)
                  {
                    zobraz_cislo((pocitadlo_radek / 20) , 6);         // vystup odpocitavani na displej
                  }                    

              }                     
          }
        dokoncene_mereni = 2;                                         // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni pres RS485
        break;

        
        case  65532:                                                  // vypis cele EEPROM (krome tech zaznamu, ktere maji datum nastaveny na 1.1.1970) - vypisuji se i SOFT formatovane zaznamy
          {
            dokoncene_mereni = 0;                                     // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres RS485

            unsigned int pocitadlo_radek = (max_zaznam_EEPROM - min_EEPROM) / delka_zaznamu;            
            for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
              {
                if (EEPROM_read_long(adresa+1) > 0)                   // vypisuji se jen zaznamy, ktere maji nejaky normalni datum
                  {
                    prazdno = false;
                    priprav_1_zaznam(adresa);
                    Serial.println(vystupni_retezec);

#ifdef pouzit_modbus
                test_485_mod();                                                                    // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
                test_485_kom();                                                                    // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif
                    if (digitalRead(pin_tl_ok) == LOW)                // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                  }

                pocitadlo_radek --;
                if (pocitadlo_radek % 20 == 0)
                  {
                    zobraz_cislo((pocitadlo_radek / 20) , 6);         // vystup odpocitavani na displej
                  }
              }                     
          }
        dokoncene_mereni = 2;                                         // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni pres RS485
        break;

        case   65531:                                                 // vypis stareho zaznamu na displej
          {
            //---------------
            if (adr_posl_zaznamu > 0)
              {                        
                boolean vyskocit_ze_zobrazeni = false;
                
                while (vyskocit_ze_zobrazeni == false)
                  {
    
                    // nalezeni adresy pred 'list' zaznamy
                    unsigned long adr_100 = adr_posl_zaznamu;
                    
                    int pocet = list;
    
                    while (pocet > 0)                                 // nekolikrat ('list' krat) se odecte delka zaznamu ...
                      {
                        pocet --;
                        adr_100 = adr_100 - delka_zaznamu;
                        if (adr_100 < min_EEPROM)                     //  ... kdyz behem toho dojde k podteceni pod zacatek oblasti s daty ...
                          {
                            adr_100 = max_zaznam_EEPROM;              // ... nastavi se adresa na posledni pouzitelny zaznam v EEPROM
                          }
                      }
    
                    if ((EEPROM_read(adr_100) & 0b10000000) > 0)      // v EEPROM se nachazi platny zaznam
                      {
                        priprav_1_zaznam(adr_100);                
    
                        if ((EEPROM_read(adr_100) & 0b01000000) > 0)  // specialni pripad zaznamu pro funkci stopky
                          {
                            switch (EEPROM_read(adr_100 + 7))
                              {
                                case 1:                                                            // START
                                  {
                                    pom_disp_scroll[4]  = 109;                                     // S
                                    pom_disp_scroll[5]  = 120;                                     // t
                                    pom_disp_scroll[6]  = 119;                                     // A
                                    pom_disp_scroll[7]  =  80;                                     // r
                                    pom_disp_scroll[8]  = 120;                                     // t
                                    pom_disp_scroll[9]  =   0;                                     // ' '
                                    pom_disp_scroll[10] =   graf_def[vystupni_retezec[31]-48];     // desitky dni
                                    pom_disp_scroll[11] =   graf_def[vystupni_retezec[32]-48] + 128;   // jednotky dni + tecka
                                    pom_disp_scroll[12] =   graf_def[vystupni_retezec[34]-48];     // desitky mesicu
                                    pom_disp_scroll[13] =   graf_def[vystupni_retezec[35]-48] + 128;   // jednotky mesicu + tecka
                                    pom_disp_scroll[14] =   0;                                     // ' '
                                    pom_disp_scroll[15] =   graf_def[vystupni_retezec[44]-48];     // desitky hodin
                                    pom_disp_scroll[16] =   graf_def[vystupni_retezec[45]-48];     // jednotky hodin
                                    pom_disp_scroll[17] =   64;                                    // '-'
                                    pom_disp_scroll[18] =   graf_def[vystupni_retezec[47]-48];     // desitky minut
                                    pom_disp_scroll[19] =   graf_def[vystupni_retezec[48]-48];     // jednotky minut
                                   
                                    vyskocit_ze_zobrazeni = disp_zaznam_scroll(20);
                                  }
                                break;
            
                                case 2:                                                            // STOP
                                  {
                                    pom_disp_scroll[4]  = 109;                                     // S
                                    pom_disp_scroll[5]  = 120;                                     // t
                                    pom_disp_scroll[6]  =  92;                                     // o
                                    pom_disp_scroll[7]  = 115;                                     // P
                                    pom_disp_scroll[8]  =   0;                                     // ' '
                                    pom_disp_scroll[9]  =   graf_def[vystupni_retezec[32]-48];     // desitky hodin
                                    pom_disp_scroll[10] =   graf_def[vystupni_retezec[33]-48];     // jednotky hodin
                                    pom_disp_scroll[11] =   64;                                    // '-'
                                    pom_disp_scroll[12] =   graf_def[vystupni_retezec[35]-48];     // desitky minut
                                    pom_disp_scroll[13] =   graf_def[vystupni_retezec[36]-48];     // jednotky minut
                                    pom_disp_scroll[14] =   64;                                    // '-'
                                    pom_disp_scroll[15] =   graf_def[vystupni_retezec[38]-48];     // desitky sekund
                                    pom_disp_scroll[16] =   graf_def[vystupni_retezec[39]-48] + 128;   // jednotky sekund + tecka
                                    pom_disp_scroll[17] =   graf_def[vystupni_retezec[48]-48];     // desetiny sekund
                                    pom_disp_scroll[18] =   graf_def[vystupni_retezec[49]-48];     // setiny sekund
                                    pom_disp_scroll[19] =   graf_def[vystupni_retezec[50]-48];     // tisiciny sekund
                                    
                                    vyskocit_ze_zobrazeni = disp_zaznam_scroll(20);                                 
                                  }
                                break;
            
                                case 3:                                                            // LAP
                                  {
                                    pom_disp_scroll[4]  =  56;                                     // L
                                    pom_disp_scroll[5]  = 119;                                     // A
                                    pom_disp_scroll[6]  = 115;                                     // P
                                    pom_disp_scroll[7]  =   0;                                     // ' '
                                    pom_disp_scroll[8]  =   graf_def[vystupni_retezec[89]-48];     // desitky z pocitadla mezicasu
                                    pom_disp_scroll[9]  =   graf_def[vystupni_retezec[90]-48];     // jednotky z pocitadla mezicasu
                                    pom_disp_scroll[10] =   0;                                     // ' '                        
                                    pom_disp_scroll[11] =   graf_def[vystupni_retezec[32]-48];     // desitky hodin
                                    pom_disp_scroll[12] =   graf_def[vystupni_retezec[33]-48];     // jednotky hodin
                                    pom_disp_scroll[13] =   64;                                    // '-'
                                    pom_disp_scroll[14] =   graf_def[vystupni_retezec[35]-48];     // desitky minut
                                    pom_disp_scroll[15] =   graf_def[vystupni_retezec[36]-48];     // jednotky minut
                                    pom_disp_scroll[16] =   64;                                    // '-'
                                    pom_disp_scroll[17] =   graf_def[vystupni_retezec[38]-48];     // desitky sekund
                                    pom_disp_scroll[18] =   graf_def[vystupni_retezec[39]-48] + 128;   // jednotky sekund + tecka
                                    pom_disp_scroll[19] =   graf_def[vystupni_retezec[48]-48];     // desetiny sekund
                                    pom_disp_scroll[20] =   graf_def[vystupni_retezec[49]-48];     // setiny sekund
                                    pom_disp_scroll[21] =   graf_def[vystupni_retezec[50]-48];     // tisiciny sekund
            
                                    vyskocit_ze_zobrazeni = disp_zaznam_scroll(22);
                                  }
                                break;
            
                                case 4:                                                            // CONT
                                  {
                                    pom_disp_scroll[4]  =  57;                                     // C
                                    pom_disp_scroll[5]  =  92;                                     // o
                                    pom_disp_scroll[6]  =  84;                                     // n
                                    pom_disp_scroll[7]  = 120;                                     // t
                                    pom_disp_scroll[8]  =   0;                                     // ' '
                                    pom_disp_scroll[9]  =   graf_def[vystupni_retezec[31]-48];     // desitky dni
                                    pom_disp_scroll[10] =   graf_def[vystupni_retezec[32]-48] + 128;   // jednotky dni + tecka
                                    pom_disp_scroll[11] =   graf_def[vystupni_retezec[34]-48];     // desitky mesicu
                                    pom_disp_scroll[12] =   graf_def[vystupni_retezec[35]-48] + 128;   // jednotky mesicu + tecka
                                    pom_disp_scroll[13] =   0;    // ' '
                                    pom_disp_scroll[14] =   graf_def[vystupni_retezec[44]-48];     // desitky hodin
                                    pom_disp_scroll[15] =   graf_def[vystupni_retezec[45]-48];     // jednotky hodin
                                    pom_disp_scroll[16] =   64;                                    // '-'
                                    pom_disp_scroll[17] =   graf_def[vystupni_retezec[47]-48];     // desitky minut
                                    pom_disp_scroll[18] =   graf_def[vystupni_retezec[48]-48];     // jednotky minut
            
                                    vyskocit_ze_zobrazeni = disp_zaznam_scroll(19);
                                  }
                                break;
                              }
                          }
        
                        //----                  
                        if ((EEPROM_read(adr_100) & 0b00100000) > 0)                               // specialni pripad zaznamu pro funkci casoveho razitka
                          {
                            pom_disp_scroll[4]  = 120;                                             // t
                            pom_disp_scroll[5]  =   4;                                             // i
                            pom_disp_scroll[6]  =  55;                                             // M
                            pom_disp_scroll[7]  = 121;                                             // E                    
                            pom_disp_scroll[8]  =   0;                                             // ' '
                            pom_disp_scroll[9]  =   graf_def[vystupni_retezec[69]-48];             // stovky z poradoveho cisla razitka
                            pom_disp_scroll[10] =   graf_def[vystupni_retezec[70]-48];             // desitky  z poradoveho cisla razitka
                            pom_disp_scroll[11] =   graf_def[vystupni_retezec[71]-48];             // jednotky z poradoveho cisla razitka
                            pom_disp_scroll[12] =   0;                                             // ' '
                            pom_disp_scroll[13] =   graf_def[vystupni_retezec[31]-48];             // desitky dni
                            pom_disp_scroll[14] =   graf_def[vystupni_retezec[32]-48] +128;        // jednotky dni s teckou
                            pom_disp_scroll[15] =   graf_def[vystupni_retezec[34]-48];             // desitky mesicu
                            pom_disp_scroll[16] =   graf_def[vystupni_retezec[35]-48] +128;        // jednotky mesicu s teckou
                            pom_disp_scroll[17] =   0;                                             // ' '
                            pom_disp_scroll[18] =   graf_def[vystupni_retezec[44]-48];             // desitky hodin
                            pom_disp_scroll[19] =   graf_def[vystupni_retezec[45]-48];             // jednotky hodin
                            pom_disp_scroll[20] =   64;                                            // '-'
                            pom_disp_scroll[21] =   graf_def[vystupni_retezec[47]-48];             // desitky minut
                            pom_disp_scroll[22] =   graf_def[vystupni_retezec[48]-48];             // jednotky minut
                            pom_disp_scroll[23] =   64;                                            // '-'
                            pom_disp_scroll[24] =   graf_def[vystupni_retezec[50]-48];             // desitky sekund
                            pom_disp_scroll[25] =   graf_def[vystupni_retezec[51]-48];             // jednotky sekund
        
                            vyskocit_ze_zobrazeni = disp_zaznam_scroll(26);
                          }
        
        
                        //----                  
                        if ((EEPROM_read(adr_100) & 0b01100000) == 0)                              // bezny zaznam svetla (nebo kalibracni zaznam)
                          {
                            pom_disp_scroll[4]  =  znacka_listu;                                   // 1 znak z poznamky (A=automat, C=kalibrace, U=tlacitko Up ....)
                            pom_disp_scroll[5]  =  0;                                              // mezera
                            
                            pom_disp_scroll[6]  =  graf_def[vystupni_retezec[31]-48];              // desitky dni
                            pom_disp_scroll[7]  =  graf_def[vystupni_retezec[32]-48] | 0b10000000; // jednotky dni s teckou
                            pom_disp_scroll[8]  =  graf_def[vystupni_retezec[34]-48];              // desitky mesicu
                            pom_disp_scroll[9]  =  graf_def[vystupni_retezec[35]-48] | 0b10000000; // jednotky mesicu s teckou
                            pom_disp_scroll[10] =  0;                                              // mezera
                            pom_disp_scroll[11] =  graf_def[vystupni_retezec[44]-48];              // desitky hodin
                            pom_disp_scroll[12] =  graf_def[vystupni_retezec[45]-48];              // jednotky hodin
                            pom_disp_scroll[13] =  64;                                             // mezi hodinami a minutami je pomlcka
                            pom_disp_scroll[14] =  graf_def[vystupni_retezec[47]-48];              // desitky minut
                            pom_disp_scroll[15] =  graf_def[vystupni_retezec[48]-48];              // jednotky minut
                            pom_disp_scroll[16] =  0;                                              // mezera
                            if (vystupni_retezec[75] == '0')   pom_disp_scroll[17] = 0;            // nula na radu desitek mag se nezobrazuje
                            else       pom_disp_scroll[17] = graf_def[vystupni_retezec[75]-48];    // kdyz jsou nejake desitky mag, tak se zobrazi
                            pom_disp_scroll[18] =  graf_def[vystupni_retezec[76]-48] | 0b10000000; // jednotky mag s teckou
                            pom_disp_scroll[19] =  graf_def[vystupni_retezec[78]-48];              // desetiny mag
                            pom_disp_scroll[20] =  graf_def[vystupni_retezec[79]-48];              // setiny mag
                            pom_disp_scroll[21] =  graf_def[vystupni_retezec[80]-48];              // tisiciny mag
                                            
                            vyskocit_ze_zobrazeni = disp_zaznam_scroll(22);
                          }
                      
                      }
                    else                          // v EEPROM neni platny zaznam (bud je tam prazdna oblast, nebo je tam nejaky drive smazany zanam - SOFTformatovany)
                      {
                        // najeti 5 pomlcek na displej
                        pom_disp_scroll[4]  = 64;                                                  // -
                        pom_disp_scroll[5]  = 64;                                                  // -
                        pom_disp_scroll[6]  = 64;                                                  // -
                        pom_disp_scroll[7]  = 64;                                                  // -
                        pom_disp_scroll[8]  = 64;                                                  // -
                        vyskocit_ze_zobrazeni = disp_zaznam_scroll(9);
                      }
        
        
                    byte pompam4 = D_pamet[4];                                                     // zapamatuje se stav displeje pro nasledujici smycku, ve ktere muze dojit k jeho zhasnuti
                    byte pompam3 = D_pamet[3];                                                     //    pri aktivaci interruptu (automat, seriova komunikace)
                    byte pompam2 = D_pamet[2];
                    byte pompam1 = D_pamet[1];
                    byte pompam0 = D_pamet[0];
        
                    
                    byte pom_pocitadlo = 0;
    
                    // dokud neni stisknuto zadne tlacitko, testuje se komunikace a na displeji je zobrazen konec posledniho rotovaneho udaje
                    while (digitalRead(pin_tl_ok) == HIGH and digitalRead(pin_tl_up) == HIGH and digitalRead(pin_tl_dn) == HIGH )
                      {                                
                        interrupty();                                                              // odskoceni na test, jestli neni nejaky pozadavek o komunikaci, nebo jestli neni cas spustit automaticke mereni
        
                        D_pamet[0] = pompam0;                                                      // po navratu z interruptu zobrazi puvodni stav displeje
                        D_pamet[1] = pompam1;
                        D_pamet[2] = pompam2;
                        D_pamet[3] = pompam3;
                        D_pamet[4] = pompam4;
                        aktualizuj_displej();
                        
                        delay(100);
                      }
    
                    // neco bylo stisknuto:
                    if (digitalRead(pin_tl_ok) == LOW)                                             // stisknuto bylo tlacitko OK
                      {
                        pom_pocitadlo = 0;
                        while (digitalRead(pin_tl_ok) == LOW)                                      // dokud se tlacitko OK neuvolni ...
                          {
                            delay(20);                                                             // ... cekej
                            pom_pocitadlo ++;
                            if (pom_pocitadlo == 50) zobraz_text(28);                              // prazdny displej v okamziku, kdy je dlouhym stiskem aktivovane opakovani rotace
                            if (pom_pocitadlo > 100) pom_pocitadlo = 100;
                          }
                        delay(20);                                                                 // odruseni zakmitu pri uvolnovani tlacitka
                        prazdno = false;
        
                        if (pom_pocitadlo < 50)                                                    // OK bylo stisknuto na delsi dobu nez sekundu ...
                          {
                            vyskocit_ze_zobrazeni = true;                                          // nastavi se znacka, ze se ma provest navrat do menu
                          }                        
                      }
                        
        
    
                    if (digitalRead(pin_tl_dn) == LOW)                                             // stisknuto bylo tlacitko DOLU
                      {
                        list ++;                                                                   // posunuti na predchozi zaznam (starsi)
                      }                                                                            //    promenna 'list' ukazuje, kolikaty zaznam do zpatku se ma zobrazit
    
                    if (digitalRead(pin_tl_up) == LOW)                                             // stisknuto bylo tlacitko NAHORU
                      {
                        list --;                                                                   // posunuti na dalsi zaznam (cerstvejsi)
                        if (list < 0)                                                              // tady musi byt kontrala na podlezeni 0 (neda se zobrazit zaznam, ktery jeste nebyl zaznamenany)
                          {
                            list = 0;                           
                            tone_X(pin_bzuk,800, 30);                                              // chybove pipnuti po dosazeno minima (posledni zaznam) - novejsi zaznam uz neni
                          }
                      }
                  }
    
                zobraz_text(28);                                                                   // prazdny displej
              }
            else
              {
                zobraz_text(29);                                                                   // "-----"
              }
          }
        break;


        case  65535:                                                                               // vypise poslednich 100 zaznamu (s testem na prechod pres vrchol pouzitelne EEPROM) 
          {
            byte pocet = 100;
            // nalezeni adresy pred 100 zaznamy
            unsigned long adr_100 = adr_dalsiho_zaznamu;

            while (pocet > 0)                                                                      // 100x se odecte delka zaznamu ...
              {
                pocet --;
                adr_100 = adr_100 - delka_zaznamu;
                if (adr_100 < min_EEPROM)                                                          //  ... kdyz behem toho dojde k podteceni pod zacatek oblasti s daty ...
                  {
                    adr_100 = max_zaznam_EEPROM;                                                   // ... nastavi se adresa na posledni pouzitelny zaznam v EEPROM
                  }
              }
            
            pocet = 100;
            dokoncene_mereni = 0;                                                                  // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni

            while (pocet > 0)
              {
#ifdef pouzit_modbus
                test_485_mod();                                                                    // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#else
                test_485_kom();                                                                    // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
#endif
                pocet --;
                adr_100 = adr_100 + delka_zaznamu;
                if (adr_100 > max_zaznam_EEPROM)
                  {
                    adr_100 = min_EEPROM;
                  }
                if ((EEPROM_read(adr_100) & 0b10000000) != 0)                                      // polozka v EEPROM je obsazena
                  {
                    prazdno = false;
                    priprav_1_zaznam(adr_100);
                    Serial.println(vystupni_retezec);

                    if (digitalRead(pin_tl_ok) == LOW)                                             // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                    zobraz_cislo( pocet , 6);                                                      // zbyvajici pocet radek na displej
                  }
              }                    
          }

        dokoncene_mereni = 2;                                                                      // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni pres RS485
        break;
        
        case  0:                                                                                   // mimo rozsah
          prazdno = false;                                                                         //  na konci nevypisuje hlaseni "Zadny ulozeny zaznam" (ani v pripade, ze by opravdu zadny zaznam na zvolene polozce neexistoval)
          Serial.print(lng167);                                                                    //             Serial.print("Zvolena polozka je mimo rozsah EEPROM (1 - ");
          Serial.print((max_EEPROM - min_EEPROM) / delka_zaznamu);
          Serial.println(')');
        break;


        default:                                                                                   // jakykoliv jiny parametr znamena vypis jedne zvolene polozky
          {
            prazdno = false;                                                                       // pri vypisu zvolene polozky se na konci nevypisuje hlaseni "Zadny ulozeny zaznam"
            unsigned long adresa = ((parametr-1) * delka_zaznamu) + min_EEPROM;                    //       (ani v pripade, ze by opravdu zadny zaznam na zvolene polozce neexistoval)      
            if (adresa <= max_zaznam_EEPROM)
              {
                priprav_1_zaznam(adresa);
                Serial.println(vystupni_retezec);
              }
            else
              {
                Serial.print(lng167);                                                              // Serial.print("Zvolena polozka je mimo rozsah EEPROM (1 - ");
                Serial.print((max_EEPROM - min_EEPROM) / delka_zaznamu);
                Serial.println(')');
              }
          }                
      }
      
    if (prazdno == true)
      {
        Serial.println(lng168);                                                                    // Serial.println("Zadny ulozeny zaznam");   
      }
  }
//----------------------------------------------



//----------------------------------------------
// do docasneho pole EEPROM posklada namerene hodnoty a aktualni cas
//  vsechny potrebne hodnoty si bere  z globalnich promennych:
//       selc, int_svetlo, slave_addr, int_infra, int_full, byte_cnf, int_teplota, int_vlhkost a int_tlak
//  vystupem je globalni 'pole_EEPROM[]'
void priprava_pole_EEPROM(void)
  {
    unsigned long tt;
                                                                      // ulozeni merenych hodnot do pole 
    pole_EEPROM[0] =  0b10000000 + selc;                              // znacka, ze je blok adres v EEPROM obsazeny (bit7='1') jestli je SEC(bit0='0'), nebo SELC(bit0='1')

    pole_EEPROM[0] = pole_EEPROM[0] | (index_cidla_svetla << 1);      // pro rozsirujici desku s vice cidly svetla se uklada do prvniho bajtu i index cidla svetla 1 az 7

    if (GPS_RTC_flag == true)
      {
        tt = GPS_temp_time;                                           // zjisteni aktualniho casu v GPS (UTC)    
      }
    else
      {
        tt = rtc_korekce();                                           // zjisteni aktualniho casu v RTC (UTC)    
      }

    GPS_RTC_flag = false;                                             // zruseni docasneho pouziti GPS casu misto RTC casu
    


    pole_EEPROM[1] =  ((unsigned long)tt >> 24) & 0xFF;               // 4 bajty pro pocet sekund od 1.1.1970 (s korekci nepresnosti a pripadnym prevodem na letni cas)
    pole_EEPROM[2] =  ((unsigned long)tt >> 16) & 0xFF;
    pole_EEPROM[3] =  ((unsigned long)tt >>  8) & 0xFF;
    pole_EEPROM[4] =  ((unsigned long)tt      ) & 0xFF;    

    pole_EEPROM[5] = (int_svetlo >> 8) & 0xFF;
    pole_EEPROM[6] = (int_svetlo     ) & 0xFF;
    
    pole_EEPROM[7] = (slave_addr << 4) + znacka ;                     // SLAVE adresa se uklada do hornich 4 bitu, spodni 1 bit pro stabilni jas; 3 bity jsou znacky (tlacitko,kalibrace,rucni spousteni, automat)

    byte indexpole = 8;

#ifdef ukladat_infra      
    pole_EEPROM[indexpole]   = (int_infra >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_infra     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_full       
    pole_EEPROM[indexpole]   = (int_full >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_full     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_contr_reg 
    pole_EEPROM[indexpole] = byte_cnf;
    indexpole = indexpole + 1;
#endif

#ifdef ukladat_teplotu    
    pole_EEPROM[indexpole]   = (int_teplota >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_teplota     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_vlhkost    
    pole_EEPROM[indexpole]   = (int_vlhkost >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_vlhkost     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_tlak       
    pole_EEPROM[indexpole]   = (int_tlak >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_tlak     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_GPS
    pole_EEPROM[indexpole]   = (GPS_lat >> 24) & 0xFF;
    pole_EEPROM[indexpole+1] = (GPS_lat >> 16) & 0xFF;
    pole_EEPROM[indexpole+2] = (GPS_lat >>  8) & 0xFF;
    pole_EEPROM[indexpole+3] = (GPS_lat      ) & 0xFF;
    pole_EEPROM[indexpole+4] = (GPS_lon >> 24) & 0xFF;
    pole_EEPROM[indexpole+5] = (GPS_lon >> 16) & 0xFF;
    pole_EEPROM[indexpole+6] = (GPS_lon >>  8) & 0xFF;
    pole_EEPROM[indexpole+7] = (GPS_lon      ) & 0xFF;
    pole_EEPROM[indexpole+8] = (GPS_alt >>  8) & 0xFF;
    pole_EEPROM[indexpole+9] = (GPS_alt      ) & 0xFF;

    if ((EEPROM_read(48) & 0b11100000) > 0)                           // pokud je aktualne prednastavene nejake pozorovaci stanoviste, zapise se do GPS bloku EEPROM na nejvyssi 3 bity
      {
        pole_EEPROM[indexpole] = (pole_EEPROM[indexpole] | (EEPROM_read(48) & 0b11100000));
      }

    indexpole = indexpole + 10;
#endif    

#ifdef ukladat_naklon       
    pole_EEPROM[indexpole]   = (int_naklon >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_naklon     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_azimut 
    pole_EEPROM[indexpole]   = (int_azimut >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_azimut     ) & 0xFF;
    indexpole = indexpole + 2;
#endif


#ifdef ukladat_astro       
    pole_EEPROM[indexpole]   = Slu_elevace;                           // posledni vypoctene astro hodnoty pred kazdym zaznamem
    pole_EEPROM[indexpole+1] = Mes_elevace;
    pole_EEPROM[indexpole+2] = Mes_osvit;
    indexpole = indexpole + 3;
#endif
 
  }
//----------------------------------------------



//----------------------------------
// reset zpusobeny zamernym pretecenim watchdogu
void SOFT_RESET()
  {
    zobraz_text(46);                                                  //  "rE5Et" 
#ifndef omezeni_funkci_4                                              // logovani na SD kartu je mozne kvuli uspore prostoru v PROGMEM vypnout 
    SD_log(999,2);
#endif
    delay(1000);
    iwdg_init(IWDG_PRE_256, 10);                                      // nastaveni WD na 50 milisekund  (10*5)
    iwdg_feed();                                                      // a okamzite obcerstveni WD
    delay(1000);                                                      // behem pauzy dojde k aktivaci WD a resetu procesoru 
    
  }
//----------------------------------------------



//----------------------------------------------
// jednoducha cervena svitilna s regulaci jasu v 5 stupnich 
void svitilna(void)
  {
    zobraz_text(29);                                                  // pomlcky na displeji "-----" 
    while (digitalRead(pin_tl_ok) == LOW)                             // do teto funkce se vlezlo stiskem tlacitka OK, tak se musi pockat, az se uvolni
      {
        delay(50);
      }
    delay(100);

    byte pamet_jasu = jas_displeje;
    jas_displeje = 7;
    int uroven_jasu = pamet_LED_svitilna;
    boolean updn = false;                                             // po kazdem stisku tlacitka nahoru nebo dolu nasleduje extra pauza kvuli zakmitum
    boolean lampa_onoff = true;
    boolean konec_lampy = false;
    while (konec_lampy == false)            
      {
        if (lampa_onoff == true)                                      // lampa je rozsvicena
          {
            if (digitalRead(pin_tl_up) == LOW)                        // kdyz je rozsviceno, tak se pomoci tlacitek nahoru a dolu muze prenastavit jas
              {
                uroven_jasu ++;
                if (uroven_jasu > 5) uroven_jasu = 5;
                updn = true;
              }

            if (digitalRead(pin_tl_dn) == LOW)
              {
                uroven_jasu --;
                if (uroven_jasu < 1) uroven_jasu = 1;
                updn = true;
              }


            for (byte segmentovka = 0; segmentovka < 5; segmentovka ++)
              {
                if (segmentovka < uroven_jasu )  D_pamet[segmentovka] = 255;
                else                             D_pamet[segmentovka] = 0;
              }
            aktualizuj_displej();            

            if(updn == true)                                          // pri zmene jasu se provadi extra pauzy proti zakmitum
              {
                delay(50);
                while((digitalRead(pin_tl_up) == LOW) or (digitalRead(pin_tl_dn) == LOW))  // ceka se, az budou obe tlacitka uvolnena
                  {
                    delay(20); 
                  }
    
                delay(50);                                            // po uvolneni tlacitek se aktualizuje displej na novou uroven jasu
                updn = false;
              }          
          
          }
        else                                                          // lampa je zhasnuta
          {
            for (byte segmentovka = 0; segmentovka < 5; segmentovka ++)    // aktualizuj displej na zhasnuty stav
              {
                D_pamet[segmentovka] = 0;
              }
            aktualizuj_displej();
          }

        if (digitalRead(pin_tl_ok) == LOW)                            // je stisknuto tlacitko OK
          {
            delay(20);
            stisk_time = millis();
            while(digitalRead(pin_tl_ok) == LOW)
              {
                delay(50);
                if (millis() - stisk_time > 500)                      // OK bylo stisknuto na delsi dobu nez pul sekundy
                  {
                    zobraz_text(29);                                  // zobrazeni pomlcek
                    pamet_LED_svitilna = uroven_jasu;                 // pred ukoncenim se zaznamena posledni nastavena hodnota
                    konec_lampy = true;                               // lampa se ukoncuje.
                  }
              }                                                       // tlacitko OK bylo uvolneno
            delay(100);
            if(lampa_onoff == true) lampa_onoff = false;              // lampa se prepne ON / OFF
            else                    lampa_onoff = true;
          }
      
      }

    jas_displeje = pamet_jasu;                                        // navratem z funkce lampy se vraci puvodni jas displeje
    
  }
//----------------------------------------------



//----------------------------------------------
//  test baterie (blikani desetinnych tecek podle stavu baterie)
void test_baterie(void)
  {
    #ifdef signal_LED                                                 // pri signalizaci pomoci LED se testuje baterie jen ve 2 urovnich
        if (millis()/100 % 20 == 1)
          {
            ADC_CR2 = ADC_CR2 | 0b1;                                  // zapnuti A/D prevodniku            
            if (analogRead(pin_AD_BAT) < bat_level_2)                 // napeti na baterii je nizke ...
              {
                blik_Y_kratky();                                      // ... blika zluta LED
              }
            else                                                      // napeti na baterii je v poradku ...
              {
                blik_R();                                             // ... blika cervena LED
              }
          }
      
    #else                                                             // pri signalizaci pomoci displeje se testuje baterie v 5 urovnich 
        if (millis()/200 % 5 == 1)
          {
            D_pamet[4] =   0;                                         // Nejdriv se 4 desetinne tecky pripravi na rozsviceni (jako by byla vybita baterie)
            D_pamet[3] = 128;                                         //    (je to jen priprava - skutecne rozsviceni nastane az na konci tohoto bloku funkci 'aktualizuj_displej')
            D_pamet[2] = 128;
            D_pamet[1] = 128;
            D_pamet[0] = 128;                                         // tecka na radu jednotek blika vzdycky - nezavisle na stavu baterie
    
            // ostatni LED se podle stavu baterie mohou pripravit do zhasnuteho stavu
            ADC_CR2 = ADC_CR2 | 0b1;                                  // zapnuti A/D prevodniku
            unsigned int napeti_baterie = analogRead(pin_AD_BAT);
            
            byte mez_bat_alarmu = 0;
            if (napeti_baterie > bat_level_3)   D_pamet[1] = 0;       // napeti na baterii je vyssi nez LEVEL 3, tecka na radu desitek se nerozsviti (vic nez 7V)
            else                                mez_bat_alarmu = 1;

            if (napeti_baterie > bat_level_2)   D_pamet[2] = 0;       // napeti na baterii je vyssi nez LEVEL 2, tecka na radu stovek se nerozsviti  (vic nez 6V)
            else                                mez_bat_alarmu = 2;

            if (napeti_baterie > bat_level_1)   D_pamet[3] = 0;       // napeti na baterii je vyssi nez LEVEL 1, tecka na radu tisicu se nerozsviti  (vic nez 5V)
            else                                mez_bat_alarmu = 3;

                                                                      // pri napeti mensim nez LEVEL_1 (mene nez 5V) zustavaji svitit vsechny 4 tecky (od radu jednotek do radu tisicu)

    
    #ifdef verzePCB2
            unsigned int napeti_ref = analogRead(pin_refu);           // pro druhou verzi plosnaku byl zaveden test referencniho napeti
            if (napeti_ref > ref_level)                               // kdyz se digity na referencnim pinu zacnou zvysovat, znamena to, ze klesa referencni (napajeci) napeti pro ADC
              {
                D_pamet[4] = 128;                                     // pak se rozblikaji vsechny LED, nezavisle na tom, jaka je zmerena hodnota na pinu AD_BAT    
                D_pamet[3] = 128;
                D_pamet[2] = 128;
                D_pamet[1] = 128;
                mez_bat_alarmu = 4;            
              }
    #endif
            alarm_bat(mez_bat_alarmu);                                // akusticky alarm pri poklesu napeti baterie mezi urovnemi
            aktualizuj_displej();                                     // Skutecne rozsviceni tecek, ktere zustaly kvuli nizkemu napeti na baterii pripravene k rozsviceni
    
          }
        if (millis()/200 % 5 == 2)                                    // po nejakem case vsechny tecky zhasnou
          {
            D_pamet[0] = 0;
            D_pamet[1] = 0;
            D_pamet[2] = 0;
            D_pamet[3] = 0;
            D_pamet[4] = 0;
    
            aktualizuj_displej();      
          }
    #endif
    ADC_CR2 = ADC_CR2 & 0xFFFFFFFE;                                   // vypnuti A/D prevodniku = uspora energie

  }
//----------------------------------------------



//----------------------------------------------
// obycejne pipnuti pomoci funkce tone(), ale s testem s moznosti uplne vypnout zvuk pomoci jednoho bitu na adrese 15
void tone_X(byte tone_pin, int tone_frq, int tone_dur)
  {
    if (mute_tone_bit == false)
      {
        tone(tone_pin , tone_frq , tone_dur); 
        delay(tone_dur+1);
        noTone(tone_pin);
      }
  }
//----------------------------------------------



//----------------------------------------------
// vytiskne do seriove linky vodorovny bargraf slozeny s pomlcek a ohraniceny svislymi carami. Za zaverecnou svislo carou je CRLF
//      priklad:    |-----------|
void pomlcky(byte pocet_pomlcek)
  {
    Serial.print('|');
    for (byte i = 0; i < pocet_pomlcek ; i ++)
      {
        Serial.print('-');
      }
    Serial.println('|');    
  }
//----------------------------------------------



//----------------------------------------------
// Varovne bzuknuti pri poklesu napeti baterie pod nastavene meze
// Bzuknuti se provadi jen 1x pro kazdou mez (od zapnuti napajeni).
// Toto varovne bzuknuti neni mozne vypnout ani promennou 'mute_tone_bit'.
// Ze uz probehlo bzuknuti na prislusne mezi je zaznamenano do globalni promenne 'alarm_bat_pamet'
void alarm_bat(byte stupen)
  {
    if (bitRead(alarm_bat_pamet , stupen) == false)
      {
        for (byte cyk = stupen; cyk > 0 ; cyk--)                      // v pripade, ze se zabzuka hned nejakym vyssim stupnem, nizsi stupne se automaticky blokuji
          {
            bitSet(alarm_bat_pamet,cyk);
          }

        for (byte cyk = 0; cyk < 2 ; cyk++)
          {
            tone(pin_bzuk, 1000, 20);
            delay(50);            
            tone(pin_bzuk, 800, 20);
            delay(50);            
          }             
#ifndef omezeni_funkci_4                                 // logovani na SD kartu je mozne kvuli uspore prostoru v PROGMEM vypnout 
        SD_log(996,stupen);
#endif
      }
  }
//----------------------------------------------



//----------------------------------------------
// prevede az tri 32-bitova cisla do jednoho 24-znakoveho HEX retezce "ID_CHAR[]"
void long32_to_HEX(uint32_t cislo, byte start_pozice)
  {
    for (byte pozice = start_pozice; pozice < start_pozice + 32 ; pozice = pozice + 4)
     {
       byte ctyrbit = cislo & 0b1111;
       if (ctyrbit < 10)        ID_char[7 - (pozice/4) + start_pozice] = ctyrbit + 48;
       else                     ID_char[7 - (pozice/4) + start_pozice] = ctyrbit + 55;        
       cislo = (cislo >> 4);
     }    
  }
//----------------------------------------------



//----------------------------------------------
// pauza s uspanim procesoru - snizeni odberu
void msleep(int n)
  {
    while(n-- > 0)
      asm("wfi");
  }
